//********************************************************************;
//*                                                                  *;
//*  Intel(R) Confidential                                           *;
//*                                                                  *;
//*  TXTINFO64   Tool                                                *;
//*                                                                  *;
//*  Copyright (c) 2009 Intel Corp.                                  *;
//*                                                                  *;
//*  This code has been developed by Intel Corporation.              *;
//*  Licensee has Intel's permission to incorporate this source code *;
//*  into their product, royalty free.  This source code may NOT be  *;
//*  redistributed to anyone without Intel's written permission.     *;
//*                                                                  *;
//*  Intel specifically disclaims all warranties, express or         *;
//*  implied, and all liability, including consequential and other   *;
//*  indirect damages, for the use of this code, including liability *;
//*  for infringement of any proprietary rights, and including the   *;
//*  warranties of merchantability and fitness for a particular      *;
//*  purpose.  Intel does not assume any responsibility for any      *;
//*  errors which may appear in this code nor any responsibility to  *;
//*  update it.                                                      *;
//*                                                                  *;
//********************************************************************;

#ifndef _TPM_INFO
#define _TPM_INFO
#include "TpmTIS.h"
#include "ShellLib.h"
/********************Function pointer to status check function******************/
typedef void (*TPM_STATUS_CHECK_PROC) ();
typedef UINT8 TPM_AUTHDATA[20];
typedef TPM_AUTHDATA TPM_ENCAUTH;
CHAR16 *ptr;
#define MAX_FILE_LENGTH 32
/***************************TPM Locality 0 Base*****************************************/
#define TPM_BASE_0	0x0FED40000
#define TPM_OWNER_AUTH 0
#define TPM_SRK_AUTH 1
/*************************Warning Flags****************************************/

#define WARNING_TPM_PD_READ		0x0000000001000000 //1<<24
#define WARNING_TPM_PD_INIT		0x0000000002000000 //1<<25
#define WARNING_TPM_PO_READ		0x0000000004000000 //1<<26
#define WARNING_TPM_PO_INIT		0x0000000008000000 //1<<27
#define WARNING_TPM_UNLOCKED	0x0000000010000000 //1<<28
#define WARNING_TPM_PCR17		0x0000000020000000 //1<<29
#define WARNING_TPM_PCR18		0x0000000040000000 //1<<30

/******************************Command Retry Count*******************************************/
#define TPM_CMD_RETRY_COUNT	1
/*******************************TPM Command Tags*************************************/
#define TPM_TAG_RQU_COMMAND			0x00C1 // A command with no authentication.
#define TPM_TAG_RQU_AUTH1_COMMAND	0x00C2 // An authenticated command with one authentication handle
#define TPM_TAG_RQU_AUTH2_COMMAND	0x00C3 // An authenticated command with two authentication handles
#define TPM_TAG_RSP_COMMAND			0x00C4 // A response from a command with no authentication
#define TPM_TAG_RSP_AUTH1_COMMAND	0x00C5 // An authenticated response with one authentication handle
#define TPM_TAG_RSP_AUTH2_COMMAND	0x00C6 // An authenticated response with two authentication handles


/****************************NVIndexes****************************************/
#define INDEX_LCP_DEF       0x50000001
#define INDEX_LCP_DEF_SIZE  54

#define INDEX_LCP_OWN       0x40000001
#define INDEX_LCP_OWN_SIZE  54

#define INDEX_AUX_2           0x50000002
#define INDEX_AUX_3         0x50000003
#define INDEX_AUX_SIZE      96

/****************************Start up types**************************************************/
#define TPM_ST_CLEAR			0x0001
#define TPM_ST_STATE			0x0002
#define TPM_ST_DEACTIVATED		0x0003

/****************************TPM Sub Capability Flags***************************************/
#define TPM_CAP_PROP_OWNER 		0x111
#define H_LEN	20
#define MAX_PCR_COUNT 18	//16 are normal and last two 

/****************************TPM Command Error Codes****************************************/
#define TPM_BASE				0
#define TPM_SUCCESS				TPM_BASE
#define TPM_VENDOR_ERROR		TPM_BASE + 0x400
#define TPM_NON_FATAL			TPM_BASE + 0x800
#define TPM_RETRY				TPM_BASE + TPM_NON_FATAL
#define TPM_INVALID_POSTINIT	TPM_BASE + 38


/***************************************TPM STATUS CHECK CODES****************************/
#define TPM_STATUS_SUCCESS	0
#define TPM_STATUS_FAILURE	1

/***************************************TPM LCP Constants*********************************/
#define LCP_POLHALG_SHA1 0
#define LCP_POLTYPE_LIST 0
#define LCP_POLTYPE_ANY 1
#define LCP_MAX_LISTS 8

/*****************************************************************************************/
#pragma pack (push,1)

#define BYTETOBINARYPATTERN "%d%d%d%d%d%d%d%d"
#define BYTETOBINARY(byte)  \
  (byte & 0x80 ? 1 : 0), \
  (byte & 0x40 ? 1 : 0), \
  (byte & 0x20 ? 1 : 0), \
  (byte & 0x10 ? 1 : 0), \
  (byte & 0x08 ? 1 : 0), \
  (byte & 0x04 ? 1 : 0), \
  (byte & 0x02 ? 1 : 0), \
  (byte & 0x01 ? 1 : 0) 

/********************************************************************/
typedef struct _TPM_NV_READVALUE_IN{
	UINT16 tag;
	UINT32 paramSize;
	UINT32 ordinal;
	UINT32 nvIndex;
	UINT32 offset;
	UINT32 dataSize;
}TPM_NV_ReadValue_In;


/********************************************************************/
typedef struct _TPM_NV_READVALUE_OUT{
	UINT16 tag;
	UINT32 paramSize;
	UINT32 returnCode;
	UINT32 dataSize;
	UINT8  data[256];
}TPM_NV_ReadValue_Out;

/********************************************************************/
typedef struct _TPM_GETCAPABILITY_IN{
	UINT16 tag;
	UINT32 paramSize;
	UINT32 ordinal;
	UINT32 capArea;
	UINT32 subCapSize;
	UINT32 subCap;
}TPM_GetCapability_In;
   
typedef struct _TPM_GETCAPABILITY_OUT{
	UINT16 tag;
	UINT32 paramSize;
	UINT32 returnCode;
	UINT32 respSize; // Length of returned capability response
	UINT8  resp[256]; //<TODO> Need to make it dynamic
}TPM_GetCapability_Out;

typedef struct _TPM_PCRREAD_IN{
	UINT16 tag;
	UINT32 paramSize;
	UINT32 ordinal;
	UINT32 pcrIndex;
}TPM_PcrRead_In;

#pragma pack (pop)

/***************TPM Tool Menu Functions****************************/
void tpmStatusMenu_1();
void tpmNVRamMenu_2();
void tpmLock_3();
void tpmTakeOwnership_4();
void tpmClearOwnership_5();
void tpmPCRFunctionMenu_6();
void tpmTISFunctionMenu_7();

//tpmStatusMenu_1 functions
void DisplayTPMStatus();

//tpmNVRamMenu_2 functions
BOOLEAN initDefineSpace( TPM_DEFINE_SPACE *defSpace );
BOOLEAN initDefStructure( DEF_STRUCT *defStruct );
BOOLEAN createDefStructure( DEF_STRUCT *defStruct );
BOOLEAN readDefStructure( DEF_STRUCT *defStruct, CHAR16 *fileName);
BOOLEAN NV_Index_Def_Compare(TPM_DEFINE_SPACE *indexDefinition, TPM_1_2_NV_DATA_PUBLIC *existingIndex);
BOOLEAN readNVDataFile(  CHAR16 *fileName, NV_DATA	*nv_data, UINT32 *size);
BOOLEAN writeDefStructure( DEF_STRUCT *defStruct );
BOOLEAN displayFileList(CHAR16* extension);
BOOLEAN displayDefFile();

//tpmLock_3 functions
BOOLEAN verifyTPMLock();

//tpmTakeOwnership_4 funcitons
UINT8 get_TPM_Auth_Value( UINT8 authType, UINT8 *AuthValue );
UINT8 TPM_OIAP( UINT32 *authHandle, UINT8 *nonceEven );
void changeEndiannessTakeOwnerShip( TPM_1_2_TAKE_OWNERSHIP *inputParams );
void fillSha1DataBuffer( UINT8 *Data, TPM_1_2_CMD_HEADER *inputHeader, TPM_1_2_TAKE_OWNERSHIP *inputParams );
UINT8 TPM_TakeOwnership( UINT8 *ownerAuthValue, UINT8 *encOwnerAuthEK, UINT8 *encSrkAuthEK, UINT32 authHandle, UINT8 *nonceOdd, TPM_1_2_PUBKEY_OUT *pubKey );

UINT8 GetTPMLocality0Access();
BOOLEAN verifyTPMPresence();

/****************************TPM Commands****************************************/
UINT8 TPM_Start_Up();
UINT8 TPM_ContinueSelfTest();
UINT8 TPM_NV_ReadValue_cmd(TPM_NV_ReadValue_Out *pTPMNVReadValOut, UINT32 u32NVIdx, UINT32 u32Offset, UINT32 u32DataSize);
UINT8 TPM_Read_Volatile_Flags();
UINT8 TPM_Read_Non_Volatile_Flags();
UINT8 TPM_Read_NV_Index_List();
UINT8 TPM_Read_NV_Index_Info(IN UINT32 nvIndex, UINT32 *dataSize, BOOLEAN verbose, TPM_1_2_NV_DATA_PUBLIC *existingIndex);
UINT8 TPM_PCR_Dump();
UINT8 TPM_Read_PCRs();
UINT8 TPM_PCR_Extend( UINT32 pcrNum, UINT8* inDigest );
UINT8 TPM_PCR_Clear( UINT32 pcrNum );
UINT8 TPM_Read_Pubek( TPM_1_2_PUBKEY_OUT *pubKey );
UINT8 TPM_NV_Read(UINT32 index, UINT32 dataSize, BOOLEAN verbose, TPM_1_3_NV_READVALUE_OUT *existingIndexData);
UINT8 TPM_NV_Define( TPM_DEFINE_SPACE *defSpace );
UINT8 TPM_NV_Index_Delete();
UINT8 TPM_NV_Write( UINT32 nvIndex, UINT32 indexOffset, UINT32 numBytes, UINT8 *buffer );

/*****************Helper Functions*****************/

//<TODO> These function are not used, but kept for the test purpose, should be removed. 
UINT16 ChangeEndiannessWordASM();
UINT32 ChangeEndiannessDWordASM();
BOOLEAN ChangeEndiannessTPMDefineSpace(TPM_DEFINE_SPACE *tpmDefineSpace);
BOOLEAN readTPMRegister(UINT16 offset, OUT UINT8 *tpmRegister);
BOOLEAN writeTPMRegister(UINT16 offset, IN UINT8 *byteValue);

/****************TPM Status Check functions********************/
UINT8 tpmRegValidStsStatusCheck(void *pBuffer);
UINT8 activeLocalityStatusCheck(void *pBuffer);
UINT8 commandReadyStatusCheck(void *pBuffer);
UINT8 stsBurstCountValidStatusCheck(void *pBuffer);
UINT8 stsValidAndExpectStatusCheck(void *pBuffer);
UINT8 stsValidStatusCheck(void *pBuffer);
UINT8 stsValidAnddataAvailStatusCheck(void *pBuffer);


CHAR8 *mystrtok(CHAR16* string, const CHAR16 *delim);
/********************************************************************/

#endif //_TPM_INFO