//********************************************************************;
//*                                                                  *;
//*  Intel(R) Confidential                                           *;
//*                                                                  *;
//*  TXTINFO64   Tool                                                *;
//*                                                                  *;
//*  Copyright (c) 2009 Intel Corp.                                  *;
//*                                                                  *;
//*  This code has been developed by Intel Corporation.              *;
//*  Licensee has Intel's permission to incorporate this source code *;
//*  into their product, royalty free.  This source code may NOT be  *;
//*  redistributed to anyone without Intel's written permission.     *;
//*                                                                  *;
//*  Intel specifically disclaims all warranties, express or         *;
//*  implied, and all liability, including consequential and other   *;
//*  indirect damages, for the use of this code, including liability *;
//*  for infringement of any proprietary rights, and including the   *;
//*  warranties of merchantability and fitness for a particular      *;
//*  purpose.  Intel does not assume any responsibility for any      *;
//*  errors which may appear in this code nor any responsibility to  *;
//*  update it.                                                      *;
//*                                                                  *;
//********************************************************************;

#include "Uefi.h"
#include "Uefilib.h"
#include "MemoryAllocationLib.h" //For AllocateZeroPool
#include "Base.h" //For OFFSET_OF Macro
#include "BaseMemoryLib.h"
#include "TPMInfo.h"
#include "TpmTIS.h"
#include "WaitAndStatusCheck.h"
#include "OutputFormat.h"
#include "Debug.h"
#include "BaseCryptLib.h"
//#include "InternalCryptLib.h"
//#include "CmdLineOptions.h"
//#include "CPUTopology.h" //just for gpCPUTopoConsts->u8ILPOnlyFlag
#define RSA_PKCS1_PADDING	1

extern EFI_SYSTEM_TABLE   *gST;

/*********************************Main Menu Functions******************************************************/

void tpmStatusMenu_1()
{
	//user input
	EFI_STATUS      Status; 
	CHAR16 *Prompt = L"> ";
	CHAR16 **Response = AllocateZeroPool(256);
	//CHAR16 command[256];
	
	
	//tokenizing
	UINT32 i = 0;
	CHAR16		*tempLine;
	CHAR16		*token = NULL;
	CHAR16 		buffer[MAX_FILE_LENGTH];
	CHAR16 		delim[MAX_FILE_LENGTH] = L" ";
	CHAR16 		*command[256];
	
	UINT32 tisStatus;
	
	for( i = 0; i < 256; i++ ) {
		command[i] = AllocateZeroPool(256);
	}
	
	while(TRUE) {
	
		Print(L"TPM Status Menu\n");
		Print(L"1: Display TPM Interface Status \n");
		Print(L"2: Display TPM Volatile flags\n");
		Print(L"3: Display TPM Non-Volatile flags\n");
		//Print(L"4: Return to Main Menu\n");
		Print(L"Q: Previous Menu\n");
		
		for( i = 0; i < 256; i++ ) {
			ZeroMem ( command[i],  sizeof(command[i]) );
		}
		
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
		//Print(L"*Response: %s\n", *Response);
		
		for (i = 1, tempLine = *Response; ; i++, tempLine = NULL) {
				token = mystrtok(tempLine, &delim);
				//Print(L"Token %s\n", token);
				StrnCpy(command[i-1], token, sizeof(token) );
				//Print(L"Token %s\n", command[i-1]);
				if (token == NULL) {
					break;
				} 
		}
		
		
		if( StrCmp(command[0], L"Q") == 0 || StrCmp(command[0], L"q") == 0 || StrCmp(command[0], L"Quit") == 0 || StrCmp(command[0], L"quit") == 0 ) {
			break;
		} else if( StrCmp(command[0], L"1") == 0 ) {
			DisplayTPMStatus();
				
		} else if( StrCmp(command[0], L"2") == 0 ) {
				
			/*Read volatile capability flags*/
			UINT8 u8CmdStatusCode;
			
			u8CmdStatusCode = TPM_Read_Volatile_Flags();
			
			if(TCG_OK != u8CmdStatusCode){
				errColor;
				Print(L"ERROR in executing TPM Read Capability Command for Volatile flags TPM_CAP_FLAG_VOLATILE, Error Code: 0x%2X\n", u8CmdStatusCode);
				norColor;			
			} 

		} else if( StrCmp(command[0], L"3") == 0 ) {
			/*Read non volatile capability flags*/
			UINT8 u8CmdStatusCode;
			
			u8CmdStatusCode = TPM_Read_Non_Volatile_Flags();
			
			if(TCG_OK != u8CmdStatusCode){
				errColor;
				Print(L"ERROR in executing TPM Read Capability Command for Non Volatile flags TPM_CAP_FLAG_VOLATILE, Error Code: 0x%2X\n", u8CmdStatusCode);
				norColor;			
			} 
	
		} else if( StrCmp(command[0], L"4") == 0 ) {
			
				
		} else {
			Print(L"Unrecognized command\n");
		}
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue,  NULL, NULL );
	}
			

}

void tpmNVRamMenu_2()
{
	//user input
	EFI_STATUS      Status; 
	CHAR16 *Prompt = L"> ";
	CHAR16 *anyKey = L"Press any key to continue.";
	CHAR16 **Response = AllocateZeroPool(256);
	//CHAR16 command[256];
	
	
	//tokenizing
	UINT32 i = 0;
	CHAR16		*tempLine;
	CHAR16		*token = NULL;
	//CHAR16 		buffer[MAX_FILE_LENGTH];
	CHAR16 		delim[MAX_FILE_LENGTH] = L" ";
	CHAR16 		*command[256];
	
	DEF_STRUCT defStruct;
	
	for( i = 0; i < 256; i++ ) {
		command[i] = AllocateZeroPool(256);
	}
	
	while(TRUE) {
		Print(L"TPM NV Ram Menu\n");
		Print(L"1: Display Existing Index values\n");
		Print(L"2: Create a new index definition file from scratch.\n");
		Print(L"3: Create a definition file from an existing Index and edit it.\n"); 
		//Print(L"4: Clone  Copy an existing definition to a new name and edit it.\n"); 
		Print(L"4: Edit an existing definition file.\n"); 
		Print(L"5: Create NV Index\n");
		Print(L"6: Verify NV Index DEF \n");
		Print(L"7: Delete NV Index\n");
		Print(L"8: Read NV Data\n"); 
		Print(L"9: Write NV Data\n"); 
		Print(L"10: Verify NV Data \n");
		Print(L"11: Display NV Index DEF\n");
		Print(L"12: Display DEF File\n");
		//Print(L"13: Return to Main Menu\n");
		Print(L"Q: Previous Menu\n");
		
		for( i = 0; i < 256; i++ ) {
			ZeroMem ( command[i],  sizeof(command[i]) );
		}
		
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
		//Print(L"*Response: %s\n", *Response);
		
		for (i = 1, tempLine = *Response; ; i++, tempLine = NULL) {
				token = mystrtok(tempLine, &delim);
				//Print(L"Token %s\n", token);
				StrnCpy(command[i-1], token, sizeof(token) );
				//Print(L"Token %s\n", command[i-1]);
				if (token == NULL) {
					break;
				} 
		}
		
		
		if( StrCmp(command[0], L"Q") == 0 || StrCmp(command[0], L"q") == 0 || StrCmp(command[0], L"Quit") == 0 || StrCmp(command[0], L"quit") == 0 ) {
			break;
		} else if( StrCmp(command[0], L"1") == 0 ) {
			TPM_Read_NV_Index_List();
			
		} else if( StrCmp(command[0], L"2") == 0 ) {
			if (!initDefStructure(&defStruct)) {
				Print("Def init Failed.\n");
				break;
			}
			
			if (!createDefStructure(&defStruct)) {
				Print("Def Create Failed.\n");
				break;
			}
			
			if (!writeDefStructure( &defStruct )) {
				Print("Def Write to disk Failed.\n");
				break;
			}
			
		} else if( StrCmp(command[0], L"3") == 0 ) {
					
			
		} else if( StrCmp(command[0], L"4") == 0 ) {
			UINT8 		u8CmdStatusCode;
			CHAR16 		**Response = AllocateZeroPool(256);
			CHAR16 		*Prompt = L"Please enter the file name to read from > ";
			
			
			Print(L"\n");
			if ( !displayFileList(L"def") ) {
				continue;
			}
			
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	
			//Print(L"*response: %s \n", *Response);
			if (StrCmp(*Response, NULL) == 0 ) {
				return FALSE;
			}
			
			Print(L"\n");
			//read the selected file
			if ( !readDefStructure( &defStruct, *Response ) ) {
				continue;
			}
			
			Print(L"\n");
			//edit the def struct
			if ( !createDefStructure( &defStruct ) ) {
				continue;
			}
			Print(L"\n");
			//write the file to disk
			if ( !writeDefStructure( &defStruct ) ) {
				continue;
			}
			
		} else if( StrCmp(command[0], L"5") == 0 ) {
			UINT8 		u8CmdStatusCode;
			CHAR16 		**Response = AllocateZeroPool(256);
			CHAR16 		*Prompt = L"Please enter the file name to read from > ";
			
			Print(L"\n");
			if ( !displayFileList(L"def") ) {
				continue;
			}
			
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	
			//Print(L"*response: %s \n", *Response);
			if (StrCmp(*Response, NULL) == 0 ) {
				return FALSE;
			}
	
			Print(L"\n");
			if ( !readDefStructure( &defStruct, *Response ) ) {
				continue;
			}
			
			u8CmdStatusCode = TPM_NV_Define( &defStruct.defSpace );
			
		} else if( StrCmp(command[0], L"6") == 0 ) {
		
			
		} else if( StrCmp(command[0], L"7") == 0 ) {
			UINT8 u8CmdStatusCode;
			
			TPM_Read_NV_Index_List();
			Print(L"\n");
			u8CmdStatusCode = TPM_NV_Index_Delete();
			
			
		} else if( StrCmp(command[0], L"8") == 0 ) {
			EFI_STATUS      Status; 
			UINT64 index;
			UINT8 u8CmdStatusCode;
			UINT32 dataSize = 0;
			TPM_1_2_NV_DATA_PUBLIC existingIndex;
			TPM_1_3_NV_READVALUE_OUT existingIndexData;
			
			TPM_Read_NV_Index_List();
			Print(L"\n");
			Print(L"Please enter index address to read e.g. 0x50000002 ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &index, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			
			TPM_Read_NV_Index_Info( index, &dataSize, FALSE, &existingIndex );
			u8CmdStatusCode = TPM_NV_Read((UINT32) index, dataSize, TRUE, &existingIndexData );
			
		} else if( StrCmp(command[0], L"9") == 0 ) {
			UINT8 u8CmdStatusCode;
			EFI_STATUS      Status; 
			CHAR16 	*Prompt = L"> ";
			CHAR16 	**Response = AllocateZeroPool(256);
			UINT64 	nvindex = 0;
			UINT64 	fileOffset = 0;
			UINT64 	indexOffset = 0;
			UINT64 	numBytes = 0;
			//UINT8	buffer[256] = {0};
			//UINT8 	buffer[54] = {0x02,0x02,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20 };
			NV_DATA	nvData;
			//UINT8 	buffer[256] = {0};
			UINT32 	size = 54;
			
			
			Print(L"Please enter index address to write to e.g. 0x50000001 ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &nvindex, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			
			displayFileList(L"dat");
			
			Print(L"Please enter file name to read from: <filename>.dat ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			if (!readNVDataFile( *Response, &nvData, &size)) {
				continue;
			}
			
			
			Print(L"Please enter file offset to read from: e.g. 0 ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &fileOffset, /*force hex*/ FALSE, /*stop at space*/ TRUE );
			
			Print(L"Please enter the number of bytes to read/write: e.g. 54 ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &numBytes, /*force hex*/ FALSE, /*stop at space*/ TRUE );
			
			Print(L"Please enter the index offset to write: e.g. 0 ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &indexOffset, /*force hex*/ FALSE, /*stop at space*/ TRUE );
			/*
			Print(L"buffer[0]: 0x%x\n", buffer[0]);
			Print(L"buffer[1]: 0x%x\n", buffer[1]);
			Print(L"buffer[2]: 0x%x\n", buffer[2]);
			Print(L"buffer[3]: 0x%x\n", buffer[3]);
			Print(L"buffer[4]: 0x%x\n", buffer[4]);
			Print(L"buffer[5]: 0x%x\n", buffer[5]);
			Print(L"buffer[6]: 0x%x\n", buffer[6]);
			Print(L"buffer[7]: 0x%x\n", buffer[7]);
			Print(L"buffer[8]: 0x%x\n", buffer[8]);
			Print(L"buffer[53]: 0x%x\n", buffer[53]);*/
			
			u8CmdStatusCode = TPM_NV_Write((UINT32)nvindex, (UINT32)indexOffset, (UINT32)numBytes, nvData.buffer);
			
		} else if( StrCmp(command[0], L"10") == 0 ) {
		
			
		} else if( StrCmp(command[0], L"11") == 0 ) {
			UINT64 index;
			UINT32 dataSize = 0;
			TPM_1_2_NV_DATA_PUBLIC existingIndex;
			
			TPM_Read_NV_Index_List();
			Print(L"\n");
			
			Print(L"Please enter index address to read e.g. 0x50000002 ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &index, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			
			TPM_Read_NV_Index_Info( index, &dataSize, TRUE, &existingIndex );
			
		} else if( StrCmp(command[0], L"12") == 0 ) {
			UINT8 u8CmdStatusCode;
			Print(L"\n");
			if ( !displayFileList(L"def") ) {
				continue;
			}
			
			if ( !displayDefFile() ){
				continue;
			}
			
		} else if( StrCmp(command[0], L"13") == 0 ) {
		
			
		} else {
			Print(L"Unrecognized command\n");
			continue;
		}
		
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue, IN anyKey, NULL );
	}
}

/*
Lock the TPM
o	Sanity check (required Indexes exist)
o	Perform Tpm_Nv_DefineSpace with Index = FFFF_FFFF and size=0
o	Set gLOCKED status
o	Display status and prompt user to continue
o	Return to MAIN MENU
*/

void tpmLock_3()
{
	EFI_STATUS      Status; 
	UINT8 u8CmdStatusCode;
	TPM_DEFINE_SPACE defineSpaceLock;
	BOOLEAN psPolicy = FALSE;
	BOOLEAN aux2 = FALSE;
	BOOLEAN aux3 = FALSE;
	
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_GETCAP_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_GETCAP_PARAMS_IN inputParams;
	TPM_1_2_PERMANENT_FLAGS *pTPMNonVolatileFlags;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	UINT32 i = 0;
	UINT32 *pTPMNVIndexList = NULL;
	
	Print(L"TPM Lock\n");
	
	if (verifyTPMLock()) {
		Print(L"TPM already locked\n");
	} else {
		Print(L"Verifying TPM required indexes\n");
		inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
		inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_GetCapability_In));
		inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_GetCapability);
		inputParams.capArea = ChangeEndiannessDWord(TPM_CAP_NV_LIST);
		inputParams.subCapSize = ChangeEndiannessDWord(0x4); //ignored
		inputParams.subCap = 0; //ignored

		InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
		InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
		InputBuffers[1].Buffer = (UINT8 *)&inputParams;
		InputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_PARAMS_IN);
		
		OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
		OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
		OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
		OutputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_OUT);
		
		u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
		
		if(TCG_OK == u8CmdStatus){
			
			outputResponse.respSize = ChangeEndiannessDWord(outputResponse.respSize);
			pTPMNVIndexList = (UINT32 *) outputResponse.resp;
			
			for ( i = 0; i < (outputResponse.respSize / 4); i++ ) {
				*pTPMNVIndexList = ChangeEndiannessDWord(*pTPMNVIndexList);
				//Print(L"  Index %d: = 0x%X\n", i, *pTPMNVIndexList);
				if( *pTPMNVIndexList == 0x50000001 ){
					psPolicy = TRUE;
				}

				if( *pTPMNVIndexList == 0x50000002 ){
					aux2 = TRUE;
				} 

				if( *pTPMNVIndexList == 0x50000003 ){
					aux3 = TRUE;
				}
				pTPMNVIndexList++;
			}	
		}
		else{
			errColor;
			DEBUG_LOG((L"ERROR : tpmLock(), error in executing TPM_GetCapability command\n"));
			norColor;
		}
	
		//check to see what indexes we got
		if( psPolicy ){
			Print(L"PS Policy at address 0x50000001 found\n");
		} else{
			Print(L"S Policy at address 0x50000001 not found\n");
			return;
		}
		
		if ( aux2 && aux3 ) {
			Print(L"Aux Index at address 0x50000002 found\n");
			Print(L"Aux Index at address 0x50000003 found\n");
			Print(L"Two aux indexes are not allowed\n");
			return;
		} else if ( aux2 ) {
			Print(L"Aux Index at address 0x50000002 found\n");
		} else if ( aux3 ) {
			Print(L"Aux Index at address 0x50000003 found\n");
		} else {
			Print(L"An Aux Index was not found\n");
			return;
		}
	
		//TODO: init define space command
		defineSpaceLock.indexValue = 0xFFFFFFFF;
		defineSpaceLock.dataSize = 0;
		u8CmdStatusCode = TPM_NV_Define( &defineSpaceLock );
		Print(L"TPM Locked\n");
	}
	
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue,  NULL, NULL );
}


/*
	4 =	Take Ownership
o	Prompt user for TPM password (by definition gOwnerAuthData will be 0)
o	Generate OwnerAuthData value (SHA1[TPM_PW])
o	Read EK
o	Encrypt OwnerAuthData value (& SrkAuthData)
o	Perform TPM_TakeOwnership
o	If success:
?	Set gOwnership_flag = 1
?	Set gOwnerAuthData = OwnerAuthData (remember for other operations)
*/
void tpmTakeOwnership_4()
{
	EFI_STATUS      Status; 
	UINT32	i = 0;
	//params for auth values
	UINT8 ownerAuthValue[20] = {0};
	UINT8 srkAuthValue[20] = {0};
	UINT8 encOwnerAuthEK[256] = {0};
	UINT8 encSrkAuthEK[256] = {0};
	
	//params for pubkey
	TPM_1_2_PUBKEY_OUT pubKey;
	
	//params for OIAP
	UINT32 authHandle = 0;
	UINT8 nonceEven[20] = {0};
	UINT8 nonceOdd[20] = {0};
	
	VOID *Rsa;
	BOOLEAN RetVal;
	TPM_1_2_KEY_PARMS pParam;
	UINT8 temp[] = {0x01, 0x00, 0x01};
	
	
	Print(L"TPM Take Ownership\n");
	
	Status = get_TPM_Auth_Value( TPM_OWNER_AUTH, ownerAuthValue );
	DumpHexValue (L"ownerAuthValue: ", (UINT8 *) ownerAuthValue, 20);
	Status = get_TPM_Auth_Value( TPM_SRK_AUTH, srkAuthValue );
	DumpHexValue (L"srkAuthValue: ", (UINT8 *) srkAuthValue, 20);
	
	TPM_Read_Pubek( &pubKey );
	//DumpHexValue (L"pubKey ", (UINT8 *) pubKey.pubKey, 256);
	Print(L"pubKey.pubKeyLength: %d\n", ChangeEndiannessDWord(pubKey.pubKeyLength));
	Print(L"pubKey.keyParams.numPrimes: %d\n", ChangeEndiannessDWord(pubKey.keyParams.numPrimes));
	Print(L"pubKey.keyParams.exponentSize: %d\n", ChangeEndiannessDWord(pubKey.keyParams.exponentSize));
	//DumpHexValue (L"pubKey.keyParams.exponent: ", (UINT8 *)(pubKey.keyParams.exponent), ChangeEndiannessDWord(pubKey.keyParams.exponentSize));
	
	TPM_OIAP( &authHandle, nonceEven );
	Print(L"authHandle: %d \n", authHandle);
	DumpHexValue (L"nonceEven: ", (UINT8 *) nonceEven, 20);
	
	//encrypt the owner auth with the EK
	// Create RSA Object; RsaNew() and RsaSetKey() is the wrapper functions in EDKII BaseCryptLib.h
	//Rsa  = RsaNew ();
	// Set N module value (retrieved from PubEk) for RSA public key;
	//RetVal = RsaSetKey (Rsa, RsaKeyN, pubKey.pubKey, ChangeEndiannessDWord(pubKey.pubKeyLength));
	// Set exponent value for RSA public key; (NOTE: Need to read PubEks parameter)
	if (pubKey.keyParams.exponentSize == 0) {
		Print(L"exponent size 0\n");
		//RetVal = RsaSetKey(Rsa, RsaKeyE, temp, 3);
	} else {
		//RetVal = RsaSetKey(Rsa, RsaKeyE, pubKey.keyParams.exponent, ChangeEndiannessDWord(pubKey.keyParams.exponentSize));
		Print(L"exponent size not zero, Yack!\n");
	}
	//int RSA_public_encrypt(int flen, unsigned char *from, unsigned char *to, RSA *rsa, int padding);
	//RSA_public_encrypt(20, (UINT8 *)ownerAuthValue,	(UINT8 *) encOwnerAuthEK, Rsa, RSA_PKCS1_PADDING);
	
	//encrypt the srkAuthValue with the EK
	// Create RSA Object; RsaNew() and RsaSetKey() is the wrapper functions in EDKII BaseCryptLib.h
	//Rsa  = RsaNew ();
	// Set N module value (retrieved from PubEk) for RSA public key;
	//RetVal = RsaSetKey (Rsa, RsaKeyN, pubKey.pubKey, pubKey.pubKeyLength);
	// Set exponent value for RSA public key; (NOTE: Need to read PubEks parameter)
	if (pubKey.keyParams.exponentSize == 0) {
		//RetVal = RsaSetKey(Rsa, RsaKeyE, temp, 3);
	} else {
		//RetVal = RsaSetKey(Rsa, RsaKeyE, pubKey.keyParams.exponent, ChangeEndiannessDWord(pubKey.keyParams.exponentSize));
		Print(L"exponent size not zero, Yack!\n");
	}
	//RSA_public_encrypt(20, (UINT8 *)srkAuthValue,	(UINT8 *) encSrkAuthEK, Rsa, RSA_PKCS1_PADDING);
	
	//generate the random nonceOdd
	for ( i = 0; i < 20; i++ ) {
		nonceOdd[i] = 0xA5;
	}
	DumpHexValue (L"encOwnerAuthEK: ", (UINT8 *) encOwnerAuthEK, 256);
	DumpHexValue (L"encSrkAuthEK: ", (UINT8 *) encSrkAuthEK, 256);
	DumpHexValue (L"nonceOdd: ", (UINT8 *) nonceOdd, 256);
	
	//send the TPM_TakeOwnership message with the owner auth hmac.
	TPM_TakeOwnership( (UINT8 *)ownerAuthValue, (UINT8 *)encOwnerAuthEK, (UINT8 *)encSrkAuthEK, authHandle, (UINT8 *)nonceOdd, (UINT8 *)nonceEven, &pubKey);
	
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue,  NULL, NULL );
}

void tpmClearOwnership_5()
{
	EFI_STATUS      Status; 
	Print(L"TPM Clear Ownership\n");

	/*
	5 =	Clear Ownership
o	If gOwnerAuthData=0 ? Prompt user for TPM password
?	Generate OwnerAuthData value (SHA1[TPM_Password])
o	Perform TPM_ClearOwnership
o	If success:
?	Set gOWNERSHIP_flag = 0
?	Set gOwnerAuthData = 0
*/
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue,  NULL, NULL );
}
void tpmPCRFunctionMenu_6()
{
	
	//user input
	EFI_STATUS      Status; 
	CHAR16 *Prompt = L"> ";
	CHAR16 **Response = AllocateZeroPool(256);
	
	//tokenizing
	UINT32 i = 0;
	CHAR16		*tempLine;
	CHAR16		*token = NULL;
	//CHAR16 		buffer[MAX_FILE_LENGTH];
	CHAR16 		delim[MAX_FILE_LENGTH] = L" ";
	CHAR16 		*command[256];
	
	for( i = 0; i < 256; i++ ) {
		command[i] = AllocateZeroPool(256);
	}
	
	while(TRUE) {
	
		Print(L"TPM PCR Functions\n");
		Print(L"1: Create PCR Dump File (i.e., capture the PCR values for use in creating a PCR Info) \n");
		Print(L"2: Display PCR values\n");
		Print(L"3: Display PCR Log\n"); 
		Print(L"4: Extend PCR (future)\n"); 
		Print(L"5: Clear PCR (future)\n"); 
		Print(L"Q: Previous Menu\n");
		
		for( i = 0; i < 256; i++ ) {
			ZeroMem ( command[i],  sizeof(command[i]) );
		}
		
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
		//Print(L"*Response: %s\n", *Response);
		
		for (i = 1, tempLine = *Response; ; i++, tempLine = NULL) {
				token = mystrtok(tempLine, &delim);
				//Print(L"Token %s\n", token);
				StrnCpy(command[i-1], token, sizeof(token) );
				//Print(L"Token %s\n", command[i-1]);
				if (token == NULL) {
					break;
				} 
		}
		
		
		if( StrCmp(command[0], L"Q") == 0 || StrCmp(command[0], L"q") == 0 || StrCmp(command[0], L"Quit") == 0 || StrCmp(command[0], L"quit") == 0 ) {
			break;
		} else if( StrCmp(command[0], L"1") == 0 ) {
			CHAR16 pcrFileName[256] = L"pcrDumpFile.dat";
			TPM_PCR_Dump(pcrFileName);
			
		} else if( StrCmp(command[0], L"2") == 0 ) {
		
			TPM_Read_PCRs();
			
		} else if( StrCmp(command[0], L"3") == 0 ) {
		
			Print(L"not yet implemented\n");
		} else if( StrCmp(command[0], L"4") == 0 ) {
			UINT32 pcrNum = 0;
			UINT8 inDigest[20] = {0};
			
			Print(L"Please enter the PCR to extend: ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &pcrNum, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			
			TPM_PCR_Extend( pcrNum, inDigest );
		} else if( StrCmp(command[0], L"5") == 0 ) {
			UINT32 pcrNum = 0;
			
			Print(L"Please enter the PCR to clear: ");
			Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
			Status = ShellConvertStringToUint64( *Response, &pcrNum, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			
			TPM_PCR_Clear( pcrNum );
		} else {
			Print(L"Unrecognized command\n");
		}
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue,  NULL, NULL );
	}


}

void tpmTISFunctionMenu_7()
{
	//user input
	EFI_STATUS      Status; 
	CHAR16 *Prompt = L"> ";
	CHAR16 **Response = AllocateZeroPool(256);
	
	//tokenizing
	UINT32 i = 0;
	CHAR16		*tempLine;
	CHAR16		*token = NULL;
	//CHAR16 		buffer[MAX_FILE_LENGTH];
	CHAR16 		delim[MAX_FILE_LENGTH] = L" ";
	CHAR16 		*command[256];
	
	for( i = 0; i < 256; i++ ) {
		command[i] = AllocateZeroPool(256);
	}
	
	while(TRUE) {
	
		Print(L"TPM TIS Functions\n");
		Print(L"1: Read  <RegAddress> \n");
		Print(L"2: Write <RegAddress> <ByteValue>\n");
		Print(L"3: Locality  <level>\n"); 
		Print(L"4: Get  <RegName>\n"); 
		Print(L"5: Set <BitName> \n"); 
		Print(L"Q: Previous Menu\n");
		
		for( i = 0; i < 256; i++ ) {
			ZeroMem ( command[i],  sizeof(command[i]) );
		}
		
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
		//Print(L"*Response: %s\n", *Response);
		
		for (i = 1, tempLine = *Response; ; i++, tempLine = NULL) {
				token = mystrtok(tempLine, &delim);
				//Print(L"Token %s\n", token);
				StrnCpy(command[i-1], token, sizeof(token) );
				//Print(L"Token %s\n", command[i-1]);
				if (token == NULL) {
					break;
				} 
		}
		
		
		if( StrCmp(command[0], L"Q") == 0 || StrCmp(command[0], L"q") == 0 || StrCmp(command[0], L"Quit") == 0 || StrCmp(command[0], L"quit") == 0 ) {
			break;
		} else if( StrCmp(command[0], L"1") == 0 ) {
		
			
		} else if( StrCmp(command[0], L"2") == 0 ) {
		
			
		} else if( StrCmp(command[0], L"3") == 0 ) {
		
			
		} else if( StrCmp(command[0], L"4") == 0 ) {
		
			
		} else if( StrCmp(command[0], L"5") == 0 ) {
			
			
		} else {
			Print(L"Unrecognized command\n");
		}
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue,  NULL, NULL );
	}

}

/*********************************End Menu Functions******************************************************/

BOOLEAN initDefineSpace(TPM_DEFINE_SPACE *defSpace)
{

	defSpace->tag = TPM_TAG_RQU_COMMAND;	
	defSpace->paramSize = 0x65;	
	defSpace->Ordinal = TPM_ORD_NV_DefineSpace;				//0xCC000000	 TPM_ORD_NV_DefineSpace
	defSpace->Tag0 = 0x18;					//1800	 Start of TPM_NV_DATA_PUBLIC33	
	defSpace->indexValue = 0x50000003;				//--in TPM order- ex:01000050
									//--Start (tdTPM_PCR_INFO_SHORT3)
									//----Start (tdTPM_PCR_SELECTION3)
	defSpace->readSizeofSelect = 3;		//0x0300	------Fixed value of 3 for this version
	defSpace->readPcrSelect[0] = 0;		//0,0,0	------End # pcrSelection - 
	defSpace->readPcrSelect[1] = 0;
	defSpace->readPcrSelect[2] = 0;
	defSpace->readLocalityAtRelease = 0x1F;	//----1F=Any Locality
	ZeroMem ( &defSpace->readDigestAtRelease,  sizeof(DIGEST) ); 

	defSpace->writeSizeofSelect = 3;	//------Fixed value of 3 for this version
	defSpace->writePcrSelect[0] = 0;		//0,0,0	------End # pcrSelection - 
	defSpace->writePcrSelect[1] = 0;
	defSpace->writePcrSelect[2] = 0;
	defSpace->writeLocalityAtRelease = 0x18;//	----1F=Any Locality //0x18 is locality 3-4
	ZeroMem ( &defSpace->writeDigestAtRelease,  sizeof(DIGEST) );
	//DumpHexValue(L"defSpace.writeDigestAtRelease:", (UINT8 *)&defStruct->defSpace.writeDigestAtRelease.Sha1, sizeof(DIGEST));
									//----End # PCR_INFO_SHORT3 (26 bytes)
									//--Start permission (tdTPM_NV_ATTRIBUTES)
	defSpace->Tag1 = 0x17;	
	defSpace->Attributes = 0;				//0	----see Table 12 NV Index Attribute Values
	defSpace->bReadSTClear = 0;  			//0	----
	defSpace->bWriteSTClear = 0;			//0	----
	defSpace->bWriteSDefine = 0;  		//0	----
									//----End TPM_NV_ATTRIBUTES (9 bytes)
	defSpace->dataSize = 0;				//0 --
									// --END of TPM_NV_DATA_PUBLIC (71 bytes)
	ZeroMem ( &defSpace->encAuthValue,  sizeof(DIGEST) );
	//DumpHexValue(L"defSpace.encAuthValue:", (UINT8 *)&defStruct->defSpace.encAuthValue.Sha1, sizeof(DIGEST));
									//End of DefineSpace parameters ( 95 bytes)
									//*/
	return TRUE;
	
}

BOOLEAN initDefStructure(DEF_STRUCT *defStruct)
{
	CHAR16 *Prompt = L"TpmNvDef";
	//TpmNvDef
	CopyMem(defStruct->fileSig, Prompt, 8);
	defStruct->majorVersion = 0x01;//	 rolls when structure changes are not backward compatible 
	defStruct->minorVersion = 0x00;	// rolls when defining reserved fields or appending additional fields
	defStruct->tpmVersion = 0x0102; //	TPM version (1.2, or 2.0)
	defStruct->reserved0[0] = 0;// reserved for future definition
	defStruct->reserved0[1] = 0;
	defStruct->reserved0[2] = 0;
	
	//initDefineSpace(&defStruct->defSpace);
	
	defStruct->defSpace.tag = TPM_TAG_RQU_COMMAND;	
	defStruct->defSpace.paramSize = 0x65;	
	defStruct->defSpace.Ordinal = TPM_ORD_NV_DefineSpace;				//0xCC000000	 TPM_ORD_NV_DefineSpace
	defStruct->defSpace.Tag0 = 0x18;					//1800	 Start of TPM_NV_DATA_PUBLIC33	
	defStruct->defSpace.indexValue = 0x50000003;				//--in TPM order- ex:01000050
									//--Start (tdTPM_PCR_INFO_SHORT3)
									//----Start (tdTPM_PCR_SELECTION3)
	defStruct->defSpace.readSizeofSelect = 3;		//0x0300	------Fixed value of 3 for this version
	defStruct->defSpace.readPcrSelect[0] = 0;		//0,0,0	------End # pcrSelection - 
	defStruct->defSpace.readPcrSelect[1] = 0;
	defStruct->defSpace.readPcrSelect[2] = 0;
	defStruct->defSpace.readLocalityAtRelease = 0x1F;	//----1F=Any Locality
	ZeroMem ( &defStruct->defSpace.readDigestAtRelease,  sizeof(DIGEST) ); 
	//DumpHexValue(L"defSpace.readDigestAtRelease:", (UINT8 *)&defStruct->defSpace.readDigestAtRelease.Sha1, sizeof(DIGEST));
									//----End # PCR_INFO_SHORT3 (26 bytes)
									//--Start (tdTPM_PCR_INFO_SHORT3)
									//----Start (tdTPM_PCR_SELECTION3)
	defStruct->defSpace.writeSizeofSelect = 3;	//------Fixed value of 3 for this version
	defStruct->defSpace.writePcrSelect[0] = 0;		//0,0,0	------End # pcrSelection - 
	defStruct->defSpace.writePcrSelect[1] = 0;
	defStruct->defSpace.writePcrSelect[2] = 0;
	defStruct->defSpace.writeLocalityAtRelease = 0x18;//	----1F=Any Locality
	ZeroMem ( &defStruct->defSpace.writeDigestAtRelease,  sizeof(DIGEST) );
	//DumpHexValue(L"defSpace.writeDigestAtRelease:", (UINT8 *)&defStruct->defSpace.writeDigestAtRelease.Sha1, sizeof(DIGEST));
									//----End # PCR_INFO_SHORT3 (26 bytes)
									//--Start permission (tdTPM_NV_ATTRIBUTES)
	defStruct->defSpace.Tag1 = 0x1700;	
	defStruct->defSpace.Attributes = 0;				//0	----see Table 12 NV Index Attribute Values
	defStruct->defSpace.bReadSTClear = 0;  			//0	----
	defStruct->defSpace.bWriteSTClear = 0;			//0	----
	defStruct->defSpace.bWriteSDefine = 0;  		//0	----
									//----End TPM_NV_ATTRIBUTES (9 bytes)
	defStruct->defSpace.dataSize = 0;				//0 --
									// --END of TPM_NV_DATA_PUBLIC (71 bytes)
	ZeroMem ( &defStruct->defSpace.encAuthValue,  sizeof(DIGEST) );
	//DumpHexValue(L"defSpace.encAuthValue:", (UINT8 *)&defStruct->defSpace.encAuthValue.Sha1, sizeof(DIGEST));
									//End of DefineSpace parameters ( 95 bytes)
								
	
	ZeroMem ( &defStruct->authValue,  sizeof(DIGEST) ); //0 plain text auth value to be encrypted
	defStruct->reserved1[0] = 0;			//0	reserved for future definition
	
	return TRUE;
}

BOOLEAN ChangeEndiannessTPMDefineSpace(TPM_DEFINE_SPACE *tpmDefineSpace)
{

	tpmDefineSpace->tag = ChangeEndiannessWord(tpmDefineSpace->tag);	
	
	tpmDefineSpace->paramSize = ChangeEndiannessDWord(tpmDefineSpace->paramSize);	
	tpmDefineSpace->Ordinal = ChangeEndiannessDWord(tpmDefineSpace->Ordinal);				//0xCC000000	 TPM_ORD_NV_DEFINESPACE
	tpmDefineSpace->Tag0 = ChangeEndiannessWord(tpmDefineSpace->Tag0);					//1800	 Start of TPM_NV_DATA_PUBLIC33	
	tpmDefineSpace->indexValue = ChangeEndiannessDWord(tpmDefineSpace->indexValue);				//--in TPM order- ex:01000050
									
	tpmDefineSpace->readSizeofSelect = ChangeEndiannessWord(tpmDefineSpace->readSizeofSelect);		//0x0300	------Fixed value of 3 for this version
	tpmDefineSpace->readPcrSelect[0] = 0;		//0,0,0	------End # pcrSelection - 
	tpmDefineSpace->readPcrSelect[1] = 0;
	tpmDefineSpace->readPcrSelect[2] = 0;
	
	tpmDefineSpace->writeSizeofSelect = ChangeEndiannessWord(tpmDefineSpace->writeSizeofSelect);	//------Fixed value of 3 for this version
	tpmDefineSpace->writePcrSelect[0] = 0;		//0,0,0	------End # pcrSelection - 
	tpmDefineSpace->writePcrSelect[1] = 0;
	tpmDefineSpace->writePcrSelect[2] = 0;
	
	tpmDefineSpace->Tag1 = ChangeEndiannessWord(tpmDefineSpace->Tag1);	
	tpmDefineSpace->Attributes = ChangeEndiannessDWord(tpmDefineSpace->Attributes);				//0	----see Table 12 NV Index Attribute Values
	tpmDefineSpace->bReadSTClear = 0;  			//0	----
	tpmDefineSpace->bWriteSTClear = 0;			//0	----
	tpmDefineSpace->bWriteSDefine = 0;  		//0	----
									//----End TPM_NV_ATTRIBUTES (9 bytes)
	tpmDefineSpace->dataSize = ChangeEndiannessDWord(tpmDefineSpace->dataSize);				//0 --
								
	return TRUE;
}

BOOLEAN createDefStructure(DEF_STRUCT *defStruct)
{
	//user input
	EFI_STATUS      Status; 
	UINT64 u64offset;
	UINT32 i;
	CHAR16 **Response = AllocateZeroPool(256);
	CHAR16	responseByte[3];
	CHAR16	responseBuffer[40];
	UINT8	*digestPointer;
	CHAR16 *Prompt = L"> ";
	
	Print(L"fileSig: %08s \n", defStruct->fileSig);
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	//Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );
	//Print(L"StrnLen(*response): %d \n", StrLen(*Response));
	Print(L"*response: %s \n", *Response);
	if (StrCmp(*Response, NULL) != 0 ) {
		ZeroMem(defStruct->fileSig, 16);
		CopyMem(defStruct->fileSig, *Response, StrLen(*Response) * 2); 	
	}
	Print(L"fileSig: %s \n", defStruct->fileSig);	
	
	Print(L"majorVersion: %d \n", defStruct->majorVersion);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->majorVersion = (UINT8) u64offset;	
		Print(L"majorVersion: %d \n", defStruct->majorVersion);	
	}
	
	Print(L"minorVersion: %d \n", defStruct->minorVersion);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->minorVersion = (UINT8) u64offset;	
		Print(L"minorVersion: %d \n", defStruct->minorVersion);
	}
	
	Print(L"tpmVersion: 0x%x \n", defStruct->tpmVersion);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->tpmVersion = (UINT16) u64offset;	
		Print(L"tpmVersion: 0x%x \n", defStruct->tpmVersion);
	}

	Print(L"reserved0[0]: %d \n", defStruct->reserved0[0]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->reserved0[0] = (UINT8) u64offset;	
		Print(L"reserved0[0]: %d \n", defStruct->reserved0[0]);
	}
	
	Print(L"reserved0[1]: %d \n", defStruct->reserved0[1]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->reserved0[1] = (UINT8) u64offset;	
		Print(L"reserved0[1]: %d \n", defStruct->reserved0[1]);
	}
	
	Print(L"reserved0[2]: %d \n", defStruct->reserved0[2]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->reserved0[2] = (UINT8) u64offset;	
		Print(L"reserved0[2]: %d \n", defStruct->reserved0[2]);
	}
	
	Print(L"reserved0[3]: %d \n", defStruct->reserved0[3]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->reserved0[3] = (UINT8) u64offset;	
		Print(L"reserved0[3]: %d \n", defStruct->reserved0[3]);
	}
	
	/***************************Begining of defSpace struct***********************************/
	Print(L"defSpace.tag: 0x%x \n",defStruct->defSpace.tag);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.tag = (UINT16) u64offset;
		Print(L"defSpace.tag: 0x%x \n",defStruct->defSpace.tag);
	}
	
	Print(L"defSpace.paramSize: 0x%x \n",defStruct->defSpace.paramSize);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.paramSize = (UINT32) u64offset;
		Print(L"defSpace.paramSize: 0x%x \n",defStruct->defSpace.paramSize);
	}
	
	Print(L"defSpace.Ordinal: 0x%x \n",defStruct->defSpace.Ordinal);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.Ordinal = (UINT32) u64offset;
		Print(L"defSpace.Ordinal: 0x%x \n",defStruct->defSpace.Ordinal);
	}
	
	Print(L"defSpace.Tag0: 0x%x \n",defStruct->defSpace.Tag0);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.Tag0 = (UINT16) u64offset;
		Print(L"defSpace.Tag0: 0x%x \n",defStruct->defSpace.Tag0);
	}
	
	Print(L"defSpace.indexValue: 0x%x \n",defStruct->defSpace.indexValue);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.indexValue = (UINT32) u64offset;
		Print(L"defSpace.indexValue: 0x%x \n",defStruct->defSpace.indexValue);
	}
	
	Print(L"defSpace.readSizeofSelect: %d \n",defStruct->defSpace.readSizeofSelect);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.readSizeofSelect = (UINT16) u64offset;
		Print(L"defSpace.readSizeofSelect: %d \n",defStruct->defSpace.readSizeofSelect);
	}
	
	Print(L"defSpace.readPcrSelect[0]: %d \n",defStruct->defSpace.readPcrSelect[0]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.readPcrSelect[0] = (UINT8) u64offset;
		Print(L"defSpace.readPcrSelect[0]: %d \n",defStruct->defSpace.readPcrSelect[0]);
	}
	
	Print(L"defSpace.readPcrSelect[1]: %d \n",defStruct->defSpace.readPcrSelect[1]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.readPcrSelect[1] = (UINT8) u64offset;
		Print(L"defSpace.readPcrSelect[1]: %d \n",defStruct->defSpace.readPcrSelect[1]);
	}
	
	Print(L"defSpace.readPcrSelect[2]: %d \n",defStruct->defSpace.readPcrSelect[2]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.readPcrSelect[2] = (UINT8) u64offset;
		Print(L"defSpace.readPcrSelect[2]: %d \n",defStruct->defSpace.readPcrSelect[2]);
	}
	
	Print(L"defSpace.readLocalityAtRelease: 0x%x \n",defStruct->defSpace.readLocalityAtRelease);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.readLocalityAtRelease = (UINT8) u64offset;
		Print(L"defSpace.readLocalityAtRelease: 0x%x \n",defStruct->defSpace.readLocalityAtRelease);
	}
	
	DumpHexValue(L"defSpace.readDigestAtRelease:", (UINT8 *)&defStruct->defSpace.readDigestAtRelease, sizeof(DIGEST));
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		CopyMem(responseBuffer, *Response, StrLen(*Response) * 2);
		digestPointer = (UINT8 *)&defStruct->defSpace.readDigestAtRelease.Sha1;			
		for ( i = 0; i < StrLen(*Response); i+=2 ) {				
	
			responseByte[0] = responseBuffer[i];
			responseByte[1] = responseBuffer[i + 1];
			responseByte[2] = CHAR_NULL;
			
			Status = ShellConvertStringToUint64(responseByte, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			Print(L"u64offset: 0x%x \n", u64offset);
			
			CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
			Print(L"digestPointer: 0x%x \n", digestPointer);	
			digestPointer++;
			u64offset = 0;
			ZeroMem(responseByte, 4);			
		}
		DumpHexValue(L"defSpace.readDigestAtRelease:", (UINT8 *)&defStruct->defSpace.readDigestAtRelease.Sha1, sizeof(DIGEST));
	}
	
	Print(L"defSpace.writeSizeofSelect: %d \n",defStruct->defSpace.writeSizeofSelect);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.writeSizeofSelect = (UINT16) u64offset;
		Print(L"defSpace.writeSizeofSelect: %d \n",defStruct->defSpace.writeSizeofSelect);
	}
	
	Print(L"defSpace.writePcrSelect[0]: %d \n",defStruct->defSpace.writePcrSelect[0]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.writePcrSelect[0] = (UINT8) u64offset;
		Print(L"defSpace.writePcrSelect[0]: %d \n",defStruct->defSpace.writePcrSelect[0]);
	}
	
	Print(L"defSpace.writePcrSelect[1]: %d \n",defStruct->defSpace.writePcrSelect[1]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.writePcrSelect[1] = (UINT8) u64offset;
		Print(L"defSpace.writePcrSelect[1]: %d \n",defStruct->defSpace.writePcrSelect[1]);
	}
	
	Print(L"defSpace.writePcrSelect[2]: %d \n",defStruct->defSpace.writePcrSelect[2]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.writePcrSelect[2] = (UINT8) u64offset;
		Print(L"defSpace.writePcrSelect[2]: %d \n",defStruct->defSpace.writePcrSelect[2]);
	}
	
	Print(L"defSpace.writeLocalityAtRelease: 0x%x \n",defStruct->defSpace.writeLocalityAtRelease);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.writeLocalityAtRelease = (UINT8) u64offset;
		Print(L"defSpace.writeLocalityAtRelease: 0x%x \n",defStruct->defSpace.writeLocalityAtRelease);
	}
		
	DumpHexValue(L"defSpace.writeDigestAtRelease:", (UINT8 *)&defStruct->defSpace.writeDigestAtRelease, sizeof(DIGEST));
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		CopyMem(responseBuffer, *Response, StrLen(*Response) * 2);
		digestPointer = (UINT8 *)&defStruct->defSpace.writeDigestAtRelease.Sha1;			
		for ( i = 0; i < StrLen(*Response); i+=2 ) {				
	
			responseByte[0] = responseBuffer[i];
			responseByte[1] = responseBuffer[i + 1];
			responseByte[2] = CHAR_NULL;
			
			Status = ShellConvertStringToUint64(responseByte, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			Print(L"u64offset: 0x%x \n", u64offset);
			
			CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
			Print(L"digestPointer: 0x%x \n", digestPointer);	
			digestPointer++;
			u64offset = 0;
			ZeroMem(responseByte, 4);			
		}
		DumpHexValue(L"defSpace.writeDigestAtRelease:", (UINT8 *)&defStruct->defSpace.writeDigestAtRelease.Sha1, sizeof(DIGEST));
	}
	
	Print(L"defSpace.Tag1: 0x%x \n",defStruct->defSpace.Tag1);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.Tag1 = (UINT16) u64offset;
		Print(L"defSpace.Tag1: 0x%x \n",defStruct->defSpace.Tag1);
	}
	
	Print(L"defSpace.Attributes: 0x%x \n",defStruct->defSpace.Attributes);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
		defStruct->defSpace.Attributes = (UINT32) u64offset;
		Print(L"defSpace.Attributes: 0x%x \n",defStruct->defSpace.Attributes);
	}
	
	Print(L"defSpace.bReadSTClear: %d \n",defStruct->defSpace.bReadSTClear);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.bReadSTClear = (UINT8) u64offset;
		Print(L"defSpace.bReadSTClear: %d \n",defStruct->defSpace.bReadSTClear);
	}
	
	Print(L"defSpace.bWriteSTClear: %d \n",defStruct->defSpace.bWriteSTClear);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.bWriteSTClear = (UINT8) u64offset;
		Print(L"defSpace.bWriteSTClear: %d \n",defStruct->defSpace.bWriteSTClear);
	}
	
	Print(L"defSpace.bWriteSDefine: %d \n",defStruct->defSpace.bWriteSDefine);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.bWriteSDefine = (UINT8) u64offset;
		Print(L"defSpace.bWriteSDefine: %d \n",defStruct->defSpace.bWriteSDefine);
	}
	
	Print(L"defSpace.dataSize: %d \n",defStruct->defSpace.dataSize);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->defSpace.dataSize = (UINT32) u64offset;
		Print(L"defSpace.dataSize: %d \n",defStruct->defSpace.dataSize);
	}
	
	DumpHexValue(L"defSpace.encAuthValue:", (UINT8 *)&defStruct->defSpace.encAuthValue, sizeof(DIGEST));
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		CopyMem(responseBuffer, *Response, StrLen(*Response) * 2);
		digestPointer = (UINT8 *)&defStruct->defSpace.encAuthValue;			
		for ( i = 0; i < StrLen(*Response); i+=2 ) {				
	
			responseByte[0] = responseBuffer[i];
			responseByte[1] = responseBuffer[i + 1];
			responseByte[2] = CHAR_NULL;
			
			Status = ShellConvertStringToUint64(responseByte, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			Print(L"u64offset: 0x%x \n", u64offset);
			
			CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
			Print(L"digestPointer: 0x%x \n", digestPointer);	
			digestPointer++;
			u64offset = 0;
			ZeroMem(responseByte, 4);			
		}
		DumpHexValue(L"defSpace.encAuthValue:", (UINT8 *)&defStruct->defSpace.encAuthValue, sizeof(DIGEST));
	}
	
	/**************************End of defSpace struct*********************************************/

	DumpHexValue(L"defStruct->authValue:", (UINT8 *)&defStruct->authValue, sizeof(DIGEST));
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		CopyMem(responseBuffer, *Response, StrLen(*Response) * 2);
		digestPointer = (UINT8 *)&defStruct->authValue;			
		for ( i = 0; i < StrLen(*Response); i+=2 ) {				
	
			responseByte[0] = responseBuffer[i];
			responseByte[1] = responseBuffer[i + 1];
			responseByte[2] = CHAR_NULL;
			
			Status = ShellConvertStringToUint64(responseByte, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			Print(L"u64offset: 0x%x \n", u64offset);
			
			CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
			Print(L"digestPointer: 0x%x \n", digestPointer);	
			digestPointer++;
			u64offset = 0;
			ZeroMem(responseByte, 4);			
		}
		DumpHexValue(L"defStruct->authValue:", (UINT8 *)&defStruct->authValue, sizeof(DIGEST));
	}
	
	Print(L"defStruct->reserved1[0]: 0x%x \n",defStruct->reserved1[0]);
	*Response = NULL;
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	if (StrCmp(*Response, NULL) != 0 ) {
		Status = ShellConvertStringToUint64( *Response, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
		defStruct->reserved1[0] = (UINT8) u64offset;	
		Print(L"defStruct->reserved1[0]: 0x%x \n",defStruct->reserved1[0]);
	}
	
	return TRUE;
}

BOOLEAN writeDefStructure( DEF_STRUCT *defStruct )
{

	EFI_STATUS      Status; 
	EFI_FILE_HANDLE	FileHandle;
	UINT32 			Size;
	CHAR16			TagBuffer[2];
	CHAR16 			fname[256];
	CHAR16 			writeBuf[256];
	UINTN   		BufferSize = sizeof(writeBuf);
	UINT64			Attributes = 0;
	CHAR16 **Response = AllocateZeroPool(256);
	CHAR16 *Prompt = L"Please enter the file name to write to > ";
	
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	
	Print(L"*response: %s \n", *Response);
	if (StrCmp(*Response, NULL) == 0 ) {
		return FALSE;
	}
	
	Status = ShellOpenFileByName( *Response, &FileHandle, EFI_FILE_MODE_READ | EFI_FILE_MODE_WRITE | EFI_FILE_MODE_CREATE, Attributes );
	if(Status != EFI_SUCCESS){
		
		switch (Status) {
			case EFI_INVALID_PARAMETER:
				Print(L"ERROR: EFI_INVALID_PARAMETER\n");
				return FALSE;
				break;
			case EFI_UNSUPPORTED:
				Print(L"ERROR: EFI_UNSUPPORTED\n");
				return FALSE;
				break;
			case EFI_NOT_FOUND:
				Print(L"ERROR: EFI_NOT_FOUND\n");
				return FALSE;
				break;
			case EFI_NO_MEDIA:
				Print(L"ERROR: EFI_NO_MEDIA\n");
				return FALSE;
				break;
			case EFI_MEDIA_CHANGED:
				Print(L"ERROR: EFI_MEDIA_CHANGED\n");
				return FALSE;
				break;
			case EFI_DEVICE_ERROR:
				Print(L"ERROR: EFI_DEVICE_ERROR\n");
				return FALSE;
				break;
			case EFI_VOLUME_CORRUPTED:
				Print(L"ERROR: EFI_VOLUME_CORRUPTED\n");
				return FALSE;
				break;
			case EFI_WRITE_PROTECTED:
				Print(L"ERROR: EFI_WRITE_PROTECTED\n");
				return FALSE;
				break;
			case EFI_ACCESS_DENIED:
				Print(L"ERROR: EFI_ACCESS_DENIED\n");
				return FALSE;
				break;
			case EFI_OUT_OF_RESOURCES:
				Print(L"ERROR: EFI_OUT_OF_RESOURCES\n");
				return FALSE;
				break;
			case EFI_VOLUME_FULL:
				Print(L"ERROR: EFI_VOLUME_FULL\n");
				return FALSE;
				break;
			default:
				Print(L"ERROR: Unknown error, cannot open file %s\n");
				return FALSE;
				break;
		}
	}
	//EFI_UNICODE_BYTE_ORDER_MARK
	//Status = FileHandlePrintLine(FileHandle, L"\0xfeff");
	Size = sizeof(CHAR16);
    TagBuffer[0] = 0xfeff;
    TagBuffer[1] = CHAR_NULL;
    ShellWriteFile(FileHandle, &Size, TagBuffer);


	
	//Print(L"fileSig: %s\n", defStruct->fileSig);
	Status = FileHandlePrintLine(FileHandle, L"File Signature: %s", defStruct->fileSig);
	
	//Print(L"majorVersion: %d\n", defStruct->majorVersion);
	Status = FileHandlePrintLine(FileHandle, L"Major Version: %Ld", defStruct->majorVersion);
	
	//Print(L"minorVersion: %d\n", defStruct->minorVersion);
	Status = FileHandlePrintLine(FileHandle, L"Minor Version: %Ld", defStruct->minorVersion);
	
	//Print(L"tpmVersion: 0x%x \n", defStruct->tpmVersion);
	Status = FileHandlePrintLine(FileHandle, L"TPM Version: %Lx", defStruct->tpmVersion);
	
	//Print(L"reserved0[0]: %d \n", defStruct->reserved0[0]);
	Status = FileHandlePrintLine(FileHandle, L"reserved0 [0]: %Ld", defStruct->reserved0[0]);
	
	//Print(L"reserved0[1]: %d \n", defStruct->reserved0[1]);
	Status = FileHandlePrintLine(FileHandle, L"reserved0 [1]: %Ld", defStruct->reserved0[1]);
	
	//Print(L"reserved0[2]: %d \n", defStruct->reserved0[2]);
	Status = FileHandlePrintLine(FileHandle, L"reserved0 [2]: %Ld", defStruct->reserved0[2]);
	
	//Print(L"reserved0[3]: %d \n", defStruct->reserved0[3]);
	Status = FileHandlePrintLine(FileHandle, L"reserved0 [3]: %Ld", defStruct->reserved0[3]);
	
	//Print(L"defSpace.tag: 0x%x \n", defStruct->defSpace.tag);
	Status = FileHandlePrintLine(FileHandle, L"Tag: %Lx", defStruct->defSpace.tag);
	
	//Print(L"defSpace.paramSize: 0x%x \n", defStruct->defSpace.paramSize);
	Status = FileHandlePrintLine(FileHandle, L"Parameter Size: %Lx", defStruct->defSpace.paramSize);
	
	//Print(L"defSpace.Ordinal: 0x%x \n", defStruct->defSpace.Ordinal);
	Status = FileHandlePrintLine(FileHandle, L"Ordinal: %Lx", defStruct->defSpace.Ordinal);
	
	//Print(L"defSpace.Tag0: 0x%x \n", defStruct->defSpace.Tag0);
	Status = FileHandlePrintLine(FileHandle, L"Define Space Tag 0: %Lx", defStruct->defSpace.Tag0);
	
	//Print(L"defSpace.indexValue: 0x%x \n", defStruct->defSpace.indexValue);
	Status = FileHandlePrintLine(FileHandle, L"Index Value: %Lx", defStruct->defSpace.indexValue);
	
	//Print(L"defSpace.readSizeofSelect: %d \n", defStruct->defSpace.readSizeofSelect);
	Status = FileHandlePrintLine(FileHandle, L"Read Size of Select: %Ld", defStruct->defSpace.readSizeofSelect);
	
	//Print(L"defSpace.readPcrSelect[0]: %d \n", defStruct->defSpace.readPcrSelect[0]);
	Status = FileHandlePrintLine(FileHandle, L"Read PCR Selct [0]: %Ld", defStruct->defSpace.readPcrSelect[0]);
	
	//Print(L"defSpace.readPcrSelect[1]: %d \n", defStruct->defSpace.readPcrSelect[1]);
	Status = FileHandlePrintLine(FileHandle, L"Read PCR Selct [1]: %Ld", defStruct->defSpace.readPcrSelect[1]);
	
	//Print(L"defSpace.readPcrSelect[2]: %d \n", defStruct->defSpace.readPcrSelect[2]);
	Status = FileHandlePrintLine(FileHandle, L"Read PCR Selct [2]: %Ld", defStruct->defSpace.readPcrSelect[2]);
	
	//Print(L"defSpace.readLocalityAtRelease: 0x%x \n", defStruct->defSpace.readLocalityAtRelease);
	Status = FileHandlePrintLine(FileHandle, L"Read Locality at Release: %Lx", defStruct->defSpace.readLocalityAtRelease);
	
	//DumpHexValue(L"defSpace.readDigestAtRelease:", (UINT8 *)&defStruct->defSpace.readDigestAtRelease, sizeof(DIGEST));
	Status = FileHandlePrintLine(FileHandle, L"Read Digest at Release:%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x%L02x", 	
														defStruct->defSpace.readDigestAtRelease.Sha1[0], 
														defStruct->defSpace.readDigestAtRelease.Sha1[1],
														defStruct->defSpace.readDigestAtRelease.Sha1[2],
														defStruct->defSpace.readDigestAtRelease.Sha1[3],
														defStruct->defSpace.readDigestAtRelease.Sha1[4],
														defStruct->defSpace.readDigestAtRelease.Sha1[5],
														defStruct->defSpace.readDigestAtRelease.Sha1[6],
														defStruct->defSpace.readDigestAtRelease.Sha1[7],
														defStruct->defSpace.readDigestAtRelease.Sha1[8],
														defStruct->defSpace.readDigestAtRelease.Sha1[9],
														defStruct->defSpace.readDigestAtRelease.Sha1[10],
														defStruct->defSpace.readDigestAtRelease.Sha1[11],
														defStruct->defSpace.readDigestAtRelease.Sha1[12],
														defStruct->defSpace.readDigestAtRelease.Sha1[13],
														defStruct->defSpace.readDigestAtRelease.Sha1[14],
														defStruct->defSpace.readDigestAtRelease.Sha1[15],
														defStruct->defSpace.readDigestAtRelease.Sha1[16],
														defStruct->defSpace.readDigestAtRelease.Sha1[17],
														defStruct->defSpace.readDigestAtRelease.Sha1[18],
														defStruct->defSpace.readDigestAtRelease.Sha1[19] );
														
	//Print(L"defSpace.writeSizeofSelect: %d \n", defStruct->defSpace.writeSizeofSelect);
	Status = FileHandlePrintLine(FileHandle, L"Write Size of Select: %Ld", defStruct->defSpace.writeSizeofSelect);
	
	//Print(L"defSpace.writePcrSelect[0]: %d \n", defStruct->defSpace.writePcrSelect[0]);
	Status = FileHandlePrintLine(FileHandle, L"Write PCR Selct [0]: %Ld", defStruct->defSpace.writePcrSelect[0]);
	
	//Print(L"defSpace.writePcrSelect[1]: %d \n", defStruct->defSpace.writePcrSelect[1]);
	Status = FileHandlePrintLine(FileHandle, L"Write PCR Selct [1]: %Ld", defStruct->defSpace.writePcrSelect[1]);
	
	//Print(L"defSpace.writePcrSelect[2]: %d \n", defStruct->defSpace.writePcrSelect[2]);
	Status = FileHandlePrintLine(FileHandle, L"Write PCR Selct [2]: %Ld", defStruct->defSpace.writePcrSelect[2]);
	
	//Print(L"defSpace.writeLocalityAtRelease: 0x%x \n", defStruct->defSpace.writeLocalityAtRelease);
	Status = FileHandlePrintLine(FileHandle, L"Write Locality at Release: %Lx", defStruct->defSpace.writeLocalityAtRelease);
	
	//DumpHexValue(L"defSpace.writeDigestAtRelease:", (UINT8 *)&defStruct->defSpace.writeDigestAtRelease, sizeof(DIGEST));
	Status = FileHandlePrintLine(FileHandle, L"Write Digest at Release: %Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx", 	
														defStruct->defSpace.writeDigestAtRelease.Sha1[0], 
														defStruct->defSpace.writeDigestAtRelease.Sha1[1],
														defStruct->defSpace.writeDigestAtRelease.Sha1[2],
														defStruct->defSpace.writeDigestAtRelease.Sha1[3],
														defStruct->defSpace.writeDigestAtRelease.Sha1[4],
														defStruct->defSpace.writeDigestAtRelease.Sha1[5],
														defStruct->defSpace.writeDigestAtRelease.Sha1[6],
														defStruct->defSpace.writeDigestAtRelease.Sha1[7],
														defStruct->defSpace.writeDigestAtRelease.Sha1[8],
														defStruct->defSpace.writeDigestAtRelease.Sha1[9],
														defStruct->defSpace.writeDigestAtRelease.Sha1[10],
														defStruct->defSpace.writeDigestAtRelease.Sha1[11],
														defStruct->defSpace.writeDigestAtRelease.Sha1[12],
														defStruct->defSpace.writeDigestAtRelease.Sha1[13],
														defStruct->defSpace.writeDigestAtRelease.Sha1[14],
														defStruct->defSpace.writeDigestAtRelease.Sha1[15],
														defStruct->defSpace.writeDigestAtRelease.Sha1[16],
														defStruct->defSpace.writeDigestAtRelease.Sha1[17],
														defStruct->defSpace.writeDigestAtRelease.Sha1[18],
														defStruct->defSpace.writeDigestAtRelease.Sha1[19] );
	
	//Print(L"defSpace.Tag1: 0x%x \n", defStruct->defSpace.Tag1);
	Status = FileHandlePrintLine(FileHandle, L"Define Space Tag 1: %Lx", defStruct->defSpace.Tag1);
	
	//Print(L"defSpace.Attributes: 0x%x \n", defStruct->defSpace.Attributes);
	Status = FileHandlePrintLine(FileHandle, L"Attributes: %Lx", defStruct->defSpace.Attributes);
	
	//Print(L"defSpace.bReadSTClear: %d \n", defStruct->defSpace.bReadSTClear);
	Status = FileHandlePrintLine(FileHandle, L"Read ST Clear: %Ld", defStruct->defSpace.bReadSTClear);
	
	//Print(L"defSpace.bWriteSTClear: %d \n", defStruct->defSpace.bWriteSTClear);
	Status = FileHandlePrintLine(FileHandle, L"Write ST Clear: %Ld", defStruct->defSpace.bWriteSTClear);
	
	//Print(L"defSpace.bWriteSDefine: %d \n", defStruct->defSpace.bWriteSDefine);
	Status = FileHandlePrintLine(FileHandle, L"Write S Define: %Ld", defStruct->defSpace.bWriteSDefine);
	
	//Print(L"defSpace.dataSize: %d \n", defStruct->defSpace.dataSize);
	Status = FileHandlePrintLine(FileHandle, L"Data Size: %Ld", defStruct->defSpace.dataSize);
	
	//DumpHexValue(L"defStruct->defSpace.encAuthValue:", (UINT8 *)&defStruct->defSpace.encAuthValue, sizeof(DIGEST));
	Status = FileHandlePrintLine(FileHandle, L"encAuthValue Value: %Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx", 	
														defStruct->defSpace.encAuthValue.Sha1[0], 
														defStruct->defSpace.encAuthValue.Sha1[1],
														defStruct->defSpace.encAuthValue.Sha1[2],
														defStruct->defSpace.encAuthValue.Sha1[3],
														defStruct->defSpace.encAuthValue.Sha1[4],
														defStruct->defSpace.encAuthValue.Sha1[5],
														defStruct->defSpace.encAuthValue.Sha1[6],
														defStruct->defSpace.encAuthValue.Sha1[7],
														defStruct->defSpace.encAuthValue.Sha1[8],
														defStruct->defSpace.encAuthValue.Sha1[9],
														defStruct->defSpace.encAuthValue.Sha1[10],
														defStruct->defSpace.encAuthValue.Sha1[11],
														defStruct->defSpace.encAuthValue.Sha1[12],
														defStruct->defSpace.encAuthValue.Sha1[13],
														defStruct->defSpace.encAuthValue.Sha1[14],
														defStruct->defSpace.encAuthValue.Sha1[15],
														defStruct->defSpace.encAuthValue.Sha1[16],
														defStruct->defSpace.encAuthValue.Sha1[17],
														defStruct->defSpace.encAuthValue.Sha1[18],
														defStruct->defSpace.encAuthValue.Sha1[19] );
														
	/**************************End of defSpace struct*********************************************/

	//DumpHexValue(L"defStruct->authValue:", (UINT8 *)&defStruct->authValue, sizeof(DIGEST));
	Status = FileHandlePrintLine(FileHandle, L"Auth Value: %Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx%Lx", 	
														defStruct->authValue.Sha1[0], 
														defStruct->authValue.Sha1[1],
														defStruct->authValue.Sha1[2],
														defStruct->authValue.Sha1[3],
														defStruct->authValue.Sha1[4],
														defStruct->authValue.Sha1[5],
														defStruct->authValue.Sha1[6],
														defStruct->authValue.Sha1[7],
														defStruct->authValue.Sha1[8],
														defStruct->authValue.Sha1[9],
														defStruct->authValue.Sha1[10],
														defStruct->authValue.Sha1[11],
														defStruct->authValue.Sha1[12],
														defStruct->authValue.Sha1[13],
														defStruct->authValue.Sha1[14],
														defStruct->authValue.Sha1[15],
														defStruct->authValue.Sha1[16],
														defStruct->authValue.Sha1[17],
														defStruct->authValue.Sha1[18],
														defStruct->authValue.Sha1[19] );
														
	
	//Print(L"defStruct->reserved1[0]: 0x%x \n", defStruct->reserved1[0]);
	Status = FileHandlePrintLine(FileHandle, L"Reserved1 [0]: %Lx", defStruct->reserved1[0]);
	
	/*
	if(Status != EFI_SUCCESS){
			Print(L"ERROR: Cannot write to file %s\n", fname);
		return EFI_SUCCESS;
		
	} else { 
		//Print(L"INFO: %d bytes written.\n", BufferSize);
	}
	*/
	//close file
	Print(L"Write Complete.\n");
	Status = ShellCloseFile(&FileHandle);
	
	return TRUE;
}

BOOLEAN readDefStructure( DEF_STRUCT *defStruct, CHAR16 *fileName)
{
	EFI_STATUS      Status; 
	EFI_FILE_HANDLE	FileHandle;
	CHAR16 			readBuffer[256];
	BOOLEAN 		ascii = TRUE;
	UINTN 			size = 256;
	UINTN   		BufferSize = sizeof(readBuffer);
	UINT64			Attributes = 0;
	CHAR16			*tempLine;
	CHAR16			*token = NULL;
	CHAR16 			delim[MAX_FILE_LENGTH] = L":";
	UINT32			i = 0;	
	UINT64 			u64offset;
	CHAR16			responseByte[3];
	CHAR16			responseBuffer[40];
	UINT8			*digestPointer;
	
	
	ZeroMem ( defStruct,  sizeof(DEF_STRUCT) );
	//Status = ShellOpenFileByName(*Response, &FileHandle, EFI_FILE_MODE_READ, 0);
	//Status = ShellOpenFileByName( L"Aux3.def", &FileHandle, EFI_FILE_MODE_READ, Attributes );
	Status = ShellOpenFileByName( fileName, &FileHandle, EFI_FILE_MODE_READ, Attributes );
	if(Status != EFI_SUCCESS){
		
		switch (Status) {
			case EFI_INVALID_PARAMETER:
				Print(L"ERROR: EFI_INVALID_PARAMETER\n");
				return FALSE;
				break;
			case EFI_UNSUPPORTED:
				Print(L"ERROR: EFI_UNSUPPORTED\n");
				return FALSE;
				break;
			case EFI_NOT_FOUND:
				Print(L"ERROR: EFI_NOT_FOUND\n");
				return FALSE;
				break;
			case EFI_NO_MEDIA:
				Print(L"ERROR: EFI_NO_MEDIA\n");
				return FALSE;
				break;
			case EFI_MEDIA_CHANGED:
				Print(L"ERROR: EFI_MEDIA_CHANGED\n");
				return FALSE;
				break;
			case EFI_DEVICE_ERROR:
				Print(L"ERROR: EFI_DEVICE_ERROR\n");
				return FALSE;
				break;
			case EFI_VOLUME_CORRUPTED:
				Print(L"ERROR: EFI_VOLUME_CORRUPTED\n");
				return FALSE;
				break;
			case EFI_WRITE_PROTECTED:
				Print(L"ERROR: EFI_WRITE_PROTECTED\n");
				return FALSE;
				break;
			case EFI_ACCESS_DENIED:
				Print(L"ERROR: EFI_ACCESS_DENIED\n");
				return FALSE;
				break;
			case EFI_OUT_OF_RESOURCES:
				Print(L"ERROR: EFI_OUT_OF_RESOURCES\n");
				return FALSE;
				break;
			case EFI_VOLUME_FULL:
				Print(L"ERROR: EFI_VOLUME_FULL\n");
				return FALSE;
				break;
			default:
				Print(L"ERROR: Unknown error, cannot open file %s\n");
				return FALSE;
				break;
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			ZeroMem(defStruct->fileSig, 16);
			CopyMem(defStruct->fileSig, token, StrLen(token) * 2); 	
		}
	}			   
		
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->majorVersion = (UINT8) u64offset;	
			//Print(L"majorVersion: %d \n", defStruct->majorVersion);	
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->minorVersion = (UINT8) u64offset;	
			//Print(L"minorVersion: %d \n", defStruct->minorVersion);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->tpmVersion = (UINT16) u64offset;	
			//Print(L"tpmVersion: 0x%x \n", defStruct->tpmVersion);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->reserved0[0] = (UINT8) u64offset;	
			//Print(L"reserved0[0]: %d \n", defStruct->reserved0[0]);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->reserved0[1] = (UINT8) u64offset;	
			//Print(L"reserved0[1]: %d \n", defStruct->reserved0[1]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->reserved0[2] = (UINT8) u64offset;	
			//Print(L"reserved0[2]: %d \n", defStruct->reserved0[2]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->reserved0[3] = (UINT8) u64offset;	
			//Print(L"reserved0[3]: %d \n", defStruct->reserved0[3]);
		}
	}
	
	/***************************Begining of defSpace struct***********************************/
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.tag = (UINT16) u64offset;
			//Print(L"defSpace.tag: 0x%x \n",defStruct->defSpace.tag);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.paramSize = (UINT32) u64offset;
			//Print(L"defSpace.paramSize: 0x%x \n",defStruct->defSpace.paramSize);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.Ordinal = (UINT32) u64offset;
			//Print(L"defSpace.Ordinal: 0x%x \n",defStruct->defSpace.Ordinal);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.Tag0 = (UINT16) u64offset;
			//Print(L"defSpace.Tag0: 0x%x \n",defStruct->defSpace.Tag0);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.indexValue = (UINT32) u64offset;
			//Print(L"defSpace.indexValue: 0x%x \n",defStruct->defSpace.indexValue);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.readSizeofSelect = (UINT16) u64offset;
			//Print(L"defSpace.readSizeofSelect: %d \n",defStruct->defSpace.readSizeofSelect);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.readPcrSelect[0] = (UINT8) u64offset;
			//Print(L"defSpace.readPcrSelect[0]: %d \n",defStruct->defSpace.readPcrSelect[0]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.readPcrSelect[1] = (UINT8) u64offset;
			//Print(L"defSpace.readPcrSelect[1]: %d \n",defStruct->defSpace.readPcrSelect[1]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.readPcrSelect[2] = (UINT8) u64offset;
			//Print(L"defSpace.readPcrSelect[2]: %d \n",defStruct->defSpace.readPcrSelect[2]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.readLocalityAtRelease = (UINT8) u64offset;
			//Print(L"defSpace.readLocalityAtRelease: 0x%x \n",defStruct->defSpace.readLocalityAtRelease);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			CopyMem(responseBuffer, token, StrLen(token) * 2);
			digestPointer = (UINT8 *)&defStruct->defSpace.readDigestAtRelease.Sha1;		
			
			for ( i = 1; i < StrLen(token); i+=2) {				
		
				responseByte[0] = responseBuffer[i];
				responseByte[1] = responseBuffer[i + 1];
				responseByte[2] = CHAR_NULL;
								
				Status = ShellConvertStringToUint64(responseByte, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );
				
				CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
				digestPointer++;
				u64offset = 0;
				ZeroMem(responseByte, 4);			
			}
			//DumpHexValue(L"defSpace.readDigestAtRelease:", (UINT8 *)&defStruct->defSpace.readDigestAtRelease, sizeof(DIGEST));
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.writeSizeofSelect = (UINT16) u64offset;
			//Print(L"defSpace.writeSizeofSelect: %d \n",defStruct->defSpace.writeSizeofSelect);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.writePcrSelect[0] = (UINT8) u64offset;
			//Print(L"defSpace.writePcrSelect[0]: %d \n",defStruct->defSpace.writePcrSelect[0]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.writePcrSelect[1] = (UINT8) u64offset;
			//Print(L"defSpace.writePcrSelect[1]: %d \n",defStruct->defSpace.writePcrSelect[1]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.writePcrSelect[2] = (UINT8) u64offset;
			//Print(L"defSpace.writePcrSelect[2]: %d \n",defStruct->defSpace.writePcrSelect[2]);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.writeLocalityAtRelease = (UINT8) u64offset;
			//Print(L"defSpace.writeLocalityAtRelease: 0x%x \n",defStruct->defSpace.writeLocalityAtRelease);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			CopyMem(responseBuffer, token, StrLen(token) * 2);
			digestPointer = (UINT8 *)&defStruct->defSpace.writeDigestAtRelease.Sha1;			
			for ( i = 1; i < StrLen(token); i+=2 ) {				
		
				responseByte[0] = responseBuffer[i];
				responseByte[1] = responseBuffer[i + 1];
				responseByte[2] = CHAR_NULL;
				
				Status = ShellConvertStringToUint64(responseByte, &u64offset, TRUE, TRUE );
				
				CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
				digestPointer++;
				u64offset = 0;
				ZeroMem(responseByte, 4);			
			}
			//DumpHexValue(L"defSpace.writeDigestAtRelease:", (UINT8 *)&defStruct->defSpace.writeDigestAtRelease.Sha1, sizeof(DIGEST));
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.Tag1 = (UINT16) u64offset;
			//Print(L"defSpace.Tag1: 0x%x \n",defStruct->defSpace.Tag1);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			defStruct->defSpace.Attributes = (UINT32) u64offset;
			//Print(L"defSpace.Attributes: 0x%x \n",defStruct->defSpace.Attributes);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.bReadSTClear = (UINT8) u64offset;
			//Print(L"defSpace.bReadSTClear: %d \n",defStruct->defSpace.bReadSTClear);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.bWriteSTClear = (UINT8) u64offset;
			//Print(L"defSpace.bWriteSTClear: %d \n",defStruct->defSpace.bWriteSTClear);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.bWriteSDefine = (UINT8) u64offset;
			//Print(L"defSpace.bWriteSDefine: %d \n",defStruct->defSpace.bWriteSDefine);
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->defSpace.dataSize = (UINT32) u64offset;
			//Print(L"defSpace.dataSize: %d \n",defStruct->defSpace.dataSize);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			CopyMem(responseBuffer, token, StrLen(token) * 2);
			digestPointer = (UINT8 *)&defStruct->defSpace.encAuthValue;			
			for ( i = 1; i < StrLen(token); i+=2 ) {				
		
				responseByte[0] = responseBuffer[i];
				responseByte[1] = responseBuffer[i + 1];
				responseByte[2] = CHAR_NULL;
				
				Status = ShellConvertStringToUint64(responseByte, &u64offset,  TRUE,  TRUE );
							
				CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
				digestPointer++;
				u64offset = 0;
				ZeroMem(responseByte, 4);			
			}
			//DumpHexValue(L"defStruct->defSpace.encAuthValue:", (UINT8 *)&defStruct->defSpace.encAuthValue, sizeof(DIGEST));
		}
	}
	
	/**************************End of defSpace struct*********************************************/
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			CopyMem(responseBuffer, token, StrLen(token) * 2);
			digestPointer = (UINT8 *)&defStruct->authValue.Sha1;			
			for ( i = 1; i < StrLen(token); i+=2 ) {				
		
				responseByte[0] = responseBuffer[i];
				responseByte[1] = responseBuffer[i + 1];
				responseByte[2] = CHAR_NULL;
				
				Status = ShellConvertStringToUint64(responseByte, &u64offset,  TRUE,  TRUE );
							
				CopyMem(digestPointer, (UINT8 *)&u64offset, 1);
				digestPointer++;
				u64offset = 0;
				ZeroMem(responseByte, 4);			
			}
			//DumpHexValue(L"defStruct->authValue:", (UINT8 *)&defStruct->authValue.Sha1, sizeof(DIGEST));
		}
	}
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			defStruct->reserved1[0] = (UINT8) u64offset;	
			//Print(L"defStruct->reserved1[0]: 0x%x \n",defStruct->reserved1[0]);
		}
	}
	/*
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, TRUE,  &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
		}
	}
	*/
	Print(L"Definition File Read Complete.\n");
	Status = ShellCloseFile(&FileHandle);
	
	return TRUE;
}


BOOLEAN displayDefFile()
{

	EFI_STATUS      Status; 
	EFI_FILE_HANDLE	FileHandle;
	CHAR16 			readBuffer[256];
	BOOLEAN 		ascii = TRUE;
	UINTN 			size = 256;
	UINT64			Attributes = 0;
	CHAR16			responseBuffer[40];
	CHAR16 			**Response = AllocateZeroPool(256);
	CHAR16 			*Prompt = L"Please enter the file name to read from > ";
	CHAR16 			*anyKey = L"Press any key to continue.";
	
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	
	if (StrCmp(*Response, NULL) == 0 ) {
		return FALSE;
	}
	
	Status = ShellOpenFileByName( *Response, &FileHandle, EFI_FILE_MODE_READ, Attributes );
	if(Status != EFI_SUCCESS){
		
		switch (Status) {
			case EFI_INVALID_PARAMETER:
				Print(L"ERROR: EFI_INVALID_PARAMETER\n");
				return FALSE;
				break;
			case EFI_UNSUPPORTED:
				Print(L"ERROR: EFI_UNSUPPORTED\n");
				return FALSE;
				break;
			case EFI_NOT_FOUND:
				Print(L"ERROR: EFI_NOT_FOUND\n");
				return FALSE;
				break;
			case EFI_NO_MEDIA:
				Print(L"ERROR: EFI_NO_MEDIA\n");
				return FALSE;
				break;
			case EFI_MEDIA_CHANGED:
				Print(L"ERROR: EFI_MEDIA_CHANGED\n");
				return FALSE;
				break;
			case EFI_DEVICE_ERROR:
				Print(L"ERROR: EFI_DEVICE_ERROR\n");
				return FALSE;
				break;
			case EFI_VOLUME_CORRUPTED:
				Print(L"ERROR: EFI_VOLUME_CORRUPTED\n");
				return FALSE;
				break;
			case EFI_WRITE_PROTECTED:
				Print(L"ERROR: EFI_WRITE_PROTECTED\n");
				return FALSE;
				break;
			case EFI_ACCESS_DENIED:
				Print(L"ERROR: EFI_ACCESS_DENIED\n");
				return FALSE;
				break;
			case EFI_OUT_OF_RESOURCES:
				Print(L"ERROR: EFI_OUT_OF_RESOURCES\n");
				return FALSE;
				break;
			case EFI_VOLUME_FULL:
				Print(L"ERROR: EFI_VOLUME_FULL\n");
				return FALSE;
				break;
			default:
				Print(L"ERROR: Unknown error, cannot open file %s\n");
				return FALSE;
				break;
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	
	/***************************Begining of defSpace struct***********************************/
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	
	//break up the output so it doesn't run off the screen
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeAnyKeyContinue, IN anyKey, NULL );
		
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	
	/**************************End of defSpace struct*********************************************/
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	Print(L"%s\n", readBuffer);
	
	Print(L"Read Complete.\n");
	Status = ShellCloseFile(&FileHandle);
	

	return TRUE;
}


BOOLEAN readNVDataFile( CHAR16 *fileName, NV_DATA *nv_data, UINT32 *bufferSize)
{
	EFI_STATUS      Status; 
	EFI_FILE_HANDLE	FileHandle;
	CHAR16 			readBuffer[256];
	BOOLEAN 		ascii = TRUE;
	UINTN 			size = 256;
	//UINTN   		BufferSize = sizeof(readBuffer);
	UINT64			Attributes = 0;
	CHAR16			*tempLine;
	CHAR16			*token = NULL;
	CHAR16 			delim[MAX_FILE_LENGTH] = L":";
	UINT32			i = 0;	
	UINT64 			u64offset;
	CHAR16			responseByte[3];
	CHAR16			responseBuffer[256] = L"";
	//UINT8			*digestPointer;
	CHAR16 			**Response = AllocateZeroPool(256);
	//CHAR16 			*Prompt = L"Please enter file name to read from: <filename>.dat ";
	UINT8 			*pnvDataBuffer;
	
	pnvDataBuffer = nv_data->buffer;
	//ZeroMem ( nv_data->buffer, *bufferSize );
	
	//Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	
	//Print(L"*response: %s \n", *Response);
//	if (StrCmp(*Response, NULL) == 0 ) {
//		return FALSE;
//	}
	
	Status = ShellOpenFileByName( fileName, &FileHandle, EFI_FILE_MODE_READ, Attributes );
	if(Status != EFI_SUCCESS){
		
		switch (Status) {
			case EFI_INVALID_PARAMETER:
				Print(L"ERROR: EFI_INVALID_PARAMETER\n");
				return FALSE;
				break;
			case EFI_UNSUPPORTED:
				Print(L"ERROR: EFI_UNSUPPORTED\n");
				return FALSE;
				break;
			case EFI_NOT_FOUND:
				Print(L"ERROR: EFI_NOT_FOUND\n");
				return FALSE;
				break;
			case EFI_NO_MEDIA:
				Print(L"ERROR: EFI_NO_MEDIA\n");
				return FALSE;
				break;
			case EFI_MEDIA_CHANGED:
				Print(L"ERROR: EFI_MEDIA_CHANGED\n");
				return FALSE;
				break;
			case EFI_DEVICE_ERROR:
				Print(L"ERROR: EFI_DEVICE_ERROR\n");
				return FALSE;
				break;
			case EFI_VOLUME_CORRUPTED:
				Print(L"ERROR: EFI_VOLUME_CORRUPTED\n");
				return FALSE;
				break;
			case EFI_WRITE_PROTECTED:
				Print(L"ERROR: EFI_WRITE_PROTECTED\n");
				return FALSE;
				break;
			case EFI_ACCESS_DENIED:
				Print(L"ERROR: EFI_ACCESS_DENIED\n");
				return FALSE;
				break;
			case EFI_OUT_OF_RESOURCES:
				Print(L"ERROR: EFI_OUT_OF_RESOURCES\n");
				return FALSE;
				break;
			case EFI_VOLUME_FULL:
				Print(L"ERROR: EFI_VOLUME_FULL\n");
				return FALSE;
				break;
			default:
				Print(L"ERROR: Unknown error, cannot open file %s\n");
				return FALSE;
				break;
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ TRUE, /*stop at space*/ TRUE );	
			nv_data->indexValue = u64offset;
			//Print(L"nv_data->indexValue: 0x%x\n", nv_data->indexValue);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			nv_data->fileOffset = u64offset;
			//Print(L"nv_data->fileOffset: %d\n", nv_data->fileOffset);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			nv_data->numBytes = u64offset;
			//Print(L"nv_data->numBytes: %d\n", nv_data->numBytes);
		}
	}
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			//Print(L"Token %s\n", token);
			Status = ShellConvertStringToUint64( token, &u64offset, /*force hex*/ FALSE, /*stop at space*/ TRUE );	
			nv_data->indexOffset = u64offset;
			//Print(L"nv_data->indexOffset: %d\n", nv_data->indexOffset);
		}
	}	
	
	*bufferSize = nv_data->numBytes;
	ZeroMem ( pnvDataBuffer, nv_data->numBytes );
	
	Status = FileHandleReadLine(FileHandle, &readBuffer, &size, /*IN BOOLEAN Truncate*/ TRUE, /*IN OUT BOOLEAN *Ascii*/ &ascii);
	for (i = 1, tempLine = &readBuffer; ; i++, tempLine = NULL) {
		token = mystrtok(tempLine, &delim);
		
		if (token == NULL) {
			break;
		} 
		if (i == 2) {
			CopyMem(responseBuffer, token, StrLen(token) * 2);
			
			for ( i = 1; i < StrLen(token); i+=2 ) {				
		
				responseByte[0] = responseBuffer[i];
				responseByte[1] = responseBuffer[i + 1];
				responseByte[2] = CHAR_NULL;
				
				Status = ShellConvertStringToUint64(responseByte, &u64offset,  TRUE,  TRUE );
							
				CopyMem(pnvDataBuffer, (UINT8 *)&u64offset, 1);
				pnvDataBuffer++;
				u64offset = 0;
				ZeroMem(responseByte, 4);			
			}
			//DumpHexValue(L"NV Data:", (UINT8 *)pnvDataBuffer, sizeof(nv_data->numBytes));
		}
	}
	
	Print(L"Data File Read Complete.\n");
	Status = ShellCloseFile(&FileHandle);
	return TRUE;
}


BOOLEAN displayFileList(CHAR16* extension)
{
	EFI_STATUS Status;
	DEF_STRUCT defStruct;
	CHAR16* pwd;
	SHELL_FILE_HANDLE   FileHandle;
	EFI_FILE_INFO *FileInfo;
	BOOLEAN       NoFile = FALSE;
	CHAR16			*tempLine;
	CHAR16			*token = NULL;
	CHAR16 			delim[MAX_FILE_LENGTH] = L".";
	UINT32			i = 0;	
	
	//get a def file list from the PWD
	pwd = ShellGetCurrentDir ( /*IN CHAR16* CONST DeviceName OPTIONAL*/ NULL);
	//Print(L"PWD: %s\n", pwd);
	
	Status = ShellOpenFileByName( pwd, &FileHandle, EFI_FILE_MODE_READ, NULL );
	if(Status != EFI_SUCCESS){
		
		switch (Status) {
			case EFI_INVALID_PARAMETER:
				Print(L"ERROR: EFI_INVALID_PARAMETER\n");
				return FALSE;
				break;
			case EFI_UNSUPPORTED:
				Print(L"ERROR: EFI_UNSUPPORTED\n");
				return FALSE;
				break;
			case EFI_NOT_FOUND:
				Print(L"ERROR: EFI_NOT_FOUND\n");
				return FALSE;
				break;
			case EFI_NO_MEDIA:
				Print(L"ERROR: EFI_NO_MEDIA\n");
				return FALSE;
				break;
			case EFI_MEDIA_CHANGED:
				Print(L"ERROR: EFI_MEDIA_CHANGED\n");
				return FALSE;
				break;
			case EFI_DEVICE_ERROR:
				Print(L"ERROR: EFI_DEVICE_ERROR\n");
				return FALSE;
				break;
			case EFI_VOLUME_CORRUPTED:
				Print(L"ERROR: EFI_VOLUME_CORRUPTED\n");
				return FALSE;
				break;
			case EFI_WRITE_PROTECTED:
				Print(L"ERROR: EFI_WRITE_PROTECTED\n");
				return FALSE;
				break;
			case EFI_ACCESS_DENIED:
				Print(L"ERROR: EFI_ACCESS_DENIED\n");
				return FALSE;
				break;
			case EFI_OUT_OF_RESOURCES:
				Print(L"ERROR: EFI_OUT_OF_RESOURCES\n");
				return FALSE;
				break;
			case EFI_VOLUME_FULL:
				Print(L"ERROR: EFI_VOLUME_FULL\n");
				return FALSE;
				break;
			default:
				Print(L"ERROR: Unknown error, cannot open file %s\n");
				return FALSE;
				break;
		}
	} else {
		Print(L"Definition files in present working directory:\n");
	}
	
	/*typedef struct {
	  UINT64    Size;
	  UINT64    FileSize;
	  UINT64    PhysicalSize;
	  EFI_TIME  CreateTime;
	  EFI_TIME  LastAccessTime;
	  EFI_TIME  ModificationTime;
	  UINT64    Attribute;
	  CHAR16    FileName[1];
	} EFI_FILE_INFO;
	*/
	Status = ShellFindFirstFile ( FileHandle, &FileInfo );
	
	if(Status != EFI_SUCCESS){
		
		switch (Status) {
			case EFI_NOT_FOUND:
				Print(L"ERROR: EFI_NOT_FOUND\n");
				return FALSE;
				break;
			case EFI_NO_MEDIA:
				Print(L"ERROR: EFI_NO_MEDIA\n");
				return FALSE;
				break;
			case EFI_DEVICE_ERROR:
				Print(L"ERROR: EFI_DEVICE_ERROR\n");
				return FALSE;
				break;
			case EFI_VOLUME_CORRUPTED:
				Print(L"ERROR: EFI_VOLUME_CORRUPTED\n");
				return FALSE;
				break;
			default:
				Print(L"ERROR: Unknown error, cannot open file %s\n");
				return FALSE;
				break;
		}
	} else {
		for (i = 1, tempLine = &FileInfo->FileName; ; i++, tempLine = NULL) {
			token = mystrtok(tempLine, &delim);
			if (token == NULL) {
				break;
			} 
			if (i == 2) {	
				//if ( StrCmp( L"def", token) == 0 || StrCmp( L"DEF", token) == 0) {
				if ( StrCmp( extension, token) == 0 /*|| StrCmp( L"DEF", token) == 0*/) {
					//Print(L"    %s.def\n", FileInfo->FileName);
					Print(L"    %s.%s\n", FileInfo->FileName, extension);
				}
			}
		}
	}
	
	while ( NoFile == FALSE ) {
		//Status = ShellFindNextFile( IN SHELL_FILE_HANDLE DirHandle, IN OUT  EFI_FILE_INFO *Buffer, IN OUT BOOLEAN *NoFile );
		Status = ShellFindNextFile( FileHandle, FileInfo, &NoFile);
		if(Status != EFI_SUCCESS) {
			Print(L"Error: ShellFindNextFile failed\n");
		}
		
		for (i = 1, tempLine = &FileInfo->FileName; ; i++, tempLine = NULL) {
			token = mystrtok(tempLine, &delim);
			if (token == NULL) {
				break;
			} 
			if (i == 2) {
				//if ( StrCmp( L"def", token) == 0 || StrCmp( L"DEF", token) == 0) {
				if ( StrCmp( extension, token) == 0 /*|| StrCmp( L"DEF", token) == 0*/ ) {
					//Print(L"    %s.def\n", FileInfo->FileName);
					Print(L"    %s.%s\n", FileInfo->FileName, extension);
				}
			}
		}	
	}
  	
	
	return TRUE;
}

BOOLEAN NV_Index_Def_Compare(TPM_DEFINE_SPACE *indexDefinition, TPM_1_2_NV_DATA_PUBLIC *existingIndex)
{
	BOOLEAN same = TRUE;

	if (indexDefinition->Tag0 != existingIndex->Tag0 ) {
		Print(L"Definition file Tag0: 0x%x\n", indexDefinition->Tag0);
		Print(L"Existing index Tag0: 0x%x\n", existingIndex->Tag0);
		same = FALSE;
	}
	if (indexDefinition->indexValue != existingIndex->indexValue ) {
		Print(L"Definition file indexValue: 0x%x\n", indexDefinition->indexValue);
		Print(L"Existing index indexValue: 0x%x\n", existingIndex->indexValue);
		same = FALSE;
	}
	if (indexDefinition->readSizeofSelect != existingIndex->readSizeofSelect ) {
		Print(L"Definition file readSizeofSelect: 0x%x\n", indexDefinition->readSizeofSelect);
		Print(L"Existing index readSizeofSelect: 0x%x\n", existingIndex->readSizeofSelect);
		same = FALSE;
	}
	if (indexDefinition->readPcrSelect[0] != existingIndex->readPcrSelect[0] ) {
		Print(L"Definition file readPcrSelect[0]: 0x%x\n", indexDefinition->readPcrSelect[0]);
		Print(L"Existing index readPcrSelect[0]: 0x%x\n", existingIndex->readPcrSelect[0]);
		same = FALSE;
	}
	if (indexDefinition->readPcrSelect[1] != existingIndex->readPcrSelect[1] ) {
		Print(L"Definition file readPcrSelect[1]: 0x%x\n", indexDefinition->readPcrSelect[1]);
		Print(L"Existing index readPcrSelect[1]: 0x%x\n", existingIndex->readPcrSelect[1]);
		same = FALSE;
	}
	if (indexDefinition->readPcrSelect[2] != existingIndex->readPcrSelect[2] ) {
		Print(L"Definition file readPcrSelect[2]: 0x%x\n", indexDefinition->readPcrSelect[2]);
		Print(L"Existing index readPcrSelect[2]: 0x%x\n", existingIndex->readPcrSelect[2]);
		same = FALSE;
	}
	if (indexDefinition->readLocalityAtRelease != existingIndex->readLocalityAtRelease ) {
		Print(L"Definition file readLocalityAtRelease: 0x%x\n", indexDefinition->readLocalityAtRelease);
		Print(L"Existing index readLocalityAtRelease: 0x%x\n", existingIndex->readLocalityAtRelease);
		same = FALSE;
	}
	if ( CompareMem ( &indexDefinition->readDigestAtRelease, &existingIndex->readDigestAtRelease, 20 ) != 0 ) {
		DumpHexValue (L"Definition file readDigestAtRelease: ", (UINT8 *) &indexDefinition->readDigestAtRelease, 20);
		DumpHexValue (L"Existing index readDigestAtRelease: ", (UINT8 *) &existingIndex->readDigestAtRelease, 20);
		same = FALSE;
	}
	if (indexDefinition->writeSizeofSelect != existingIndex->writeSizeofSelect ) {
		Print(L"Definition file writeSizeofSelect: 0x%x\n", indexDefinition->writeSizeofSelect);
		Print(L"Existing index writeSizeofSelect: 0x%x\n", existingIndex->writeSizeofSelect);
		same = FALSE;
	}
	if (indexDefinition->writePcrSelect[0] != existingIndex->writePcrSelect[0] ) {
		Print(L"Definition file writePcrSelect[0]: 0x%x\n", indexDefinition->writePcrSelect[0]);
		Print(L"Existing index writePcrSelect[0]: 0x%x\n", existingIndex->writePcrSelect[0]);
		same = FALSE;
	}
	if (indexDefinition->writePcrSelect[1] != existingIndex->writePcrSelect[1] ) {
		Print(L"Definition file writePcrSelect[1]: 0x%x\n", indexDefinition->writePcrSelect[1]);
		Print(L"Existing index writePcrSelect[1]: 0x%x\n", existingIndex->writePcrSelect[1]);
		same = FALSE;
	}
	if (indexDefinition->writePcrSelect[2] != existingIndex->writePcrSelect[2] ) {
		Print(L"Definition file writePcrSelect[2]: 0x%x\n", indexDefinition->writePcrSelect[2]);
		Print(L"Existing index writePcrSelect[2]: 0x%x\n", existingIndex->writePcrSelect[2]);
		same = FALSE;
	}
	
	if (indexDefinition->writeLocalityAtRelease != existingIndex->writeLocalityAtRelease ) {
		Print(L"Definition file writeLocalityAtRelease: 0x%x\n", indexDefinition->writeLocalityAtRelease);
		Print(L"Existing index writeLocalityAtRelease: 0x%x\n", existingIndex->writeLocalityAtRelease);
		same = FALSE;
	}
	
	if ( CompareMem ( &indexDefinition->writeDigestAtRelease, &existingIndex->writeDigestAtRelease, 20 ) != 0 ) {
		DumpHexValue (L"Definition file writeDigestAtRelease: ", (UINT8 *) &indexDefinition->writeDigestAtRelease, 20);
		DumpHexValue (L"Existing index writeDigestAtRelease: ", (UINT8 *) &existingIndex->writeDigestAtRelease, 20);
		same = FALSE;
	}
	if (indexDefinition->Tag1 != existingIndex->Tag1 ) {
		Print(L"Definition file Tag1: 0x%x\n", indexDefinition->Tag1);
		Print(L"Existing index Tag1: 0x%x\n", existingIndex->Tag1);
		same = FALSE;
	}
	if (indexDefinition->Attributes != existingIndex->Attributes ) {
		Print(L"Definition file Attributes: 0x%x\n", indexDefinition->Attributes);
		Print(L"Existing index Attributes: 0x%x\n", existingIndex->Attributes);
		same = FALSE;
	}
	if (indexDefinition->bReadSTClear != existingIndex->bReadSTClear ) {
		Print(L"Definition file bReadSTClear: 0x%x\n", indexDefinition->bReadSTClear);
		Print(L"Existing index bReadSTClear: 0x%x\n", existingIndex->bReadSTClear);
		same = FALSE;
	}
	if (indexDefinition->bWriteSTClear != existingIndex->bWriteSTClear ) {
		Print(L"Definition file bWriteSTClear: 0x%x\n", indexDefinition->bWriteSTClear);
		Print(L"Existing index bWriteSTClear: 0x%x\n", existingIndex->bWriteSTClear);
		same = FALSE;
	}
	if (indexDefinition->bWriteSDefine != existingIndex->bWriteSDefine ) {
		Print(L"Definition file bWriteSDefine: 0x%x\n", indexDefinition->bWriteSDefine);
		Print(L"Existing index bWriteSDefine: 0x%x\n", existingIndex->bWriteSDefine);
		same = FALSE;
	}
	if (indexDefinition->dataSize != existingIndex->dataSize ) {
		Print(L"Definition file dataSize: 0x%x\n", indexDefinition->dataSize);
		Print(L"Existing index dataSize: 0x%x\n", existingIndex->dataSize);
		same = FALSE;
	}
	if ( !same ) {
		return FALSE;
	}
	
}

/*********************************TPM Helper Functions****************************************************/


/*	Return: TPM_STATUS_FAILURE => Either locality 0 is not accessible or request to use locality is not successfull.
TPM_STATUS_SUCCESS => Either locality 1 is already active or request to use locality is successfull.
*/
UINT8 GetTPMLocality0Access()
{
	UINT8 u8Status = TPM_STATUS_FAILURE; //No access
	TPM *pTPM = (TPM *) TPM_BASE_0; //TPM Locality 0 base address
	UINT8 u8TPM_ACCESS = 0;

	/*TODO Lakshman, check how 1000 is equivalent to 750 milisecond*/
	if(TPM_STATUS_FAILURE == WaitAndStatusCheck(1000, tpmRegValidStsStatusCheck, NULL)){
		DEBUG_LOG((L"TPM is broken, shouldn't be used\n"));
		u8Status = TPM_STATUS_FAILURE; //No access
	}
	else{
		/*check TPM_ACCESS_0.activeLocality bit[5], 0 => This locality is not active,1 => This locality is active.*/
		if(pTPM->ACCESS & (0x1 << 5)){
			//DEBUG_LOG((L"TPM_ACCESS_0.activeLocality is 1 i.e. locality 0 is already active\n"));
			u8Status = TPM_STATUS_SUCCESS; // Locality is active and we can use it
		}
		else{
			DEBUG_LOG((L"Setting TPM_ACCESS_0.requestUse for request to use locality 0\n"));
			//Set TPM_ACCESS_0.requestUse bit - request to use locality 0
			u8TPM_ACCESS = pTPM->ACCESS;
			//u8TPM_ACCESS |= (0x1 << 1); //Set TPM_ACCESS_0.requestUse bit[1]
			pTPM->ACCESS = (0x1 << 1); //Set TPM_ACCESS_0.requestUse bit[1]
			//pTPM->ACCESS = u8TPM_ACCESS;

			//Wait with timeout TIMEOUT_A (750 millisecond) or locality becomes active
			u8Status = WaitAndStatusCheck(1000, activeLocalityStatusCheck, NULL);
		}
	}
	return u8Status;
}

UINT8 TPM_Start_Up()
{

	UINT8 u8CmdStatus = TCG_OK;
	
	TPM_1_2_RET_HEADER 	outputHeader;
	UINT32 				outputResponse;
	TPM_1_2_CMD_HEADER 	inputHeader;
	UINT16 				tpmStartupType;
	TIS_BUFFER_INFO 	InputBuffers[2];
	TIS_BUFFER_INFO 	OutputBuffers[2];
	UINT32 				NumInputBuffers = 2;
	UINT32 				NumOutputBuffers = 2;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord( sizeof(TPM_1_2_CMD_HEADER) + sizeof(UINT16) );
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_Startup);
	tpmStartupType = TPM_ST_CLEAR;

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&tpmStartupType;
	InputBuffers[1].Size = sizeof(UINT16);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(UINT32);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_Start_Up: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}		
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_Start_Up(), error in executing TPM_Start_Up command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}

	return u8CmdStatus;
}

UINT8 TPM_ContinueSelfTest()
{

	UINT8 u8CmdStatus = TCG_OK;
	
	TPM_1_2_RET_HEADER 	outputHeader;
	UINT32 				outputResponse;
	TPM_1_2_CMD_HEADER 	inputHeader;
	TIS_BUFFER_INFO 	InputBuffers[2];
	TIS_BUFFER_INFO 	OutputBuffers[2];
	UINT32 				NumInputBuffers = 1;
	UINT32 				NumOutputBuffers = 2;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord( sizeof(TPM_1_2_CMD_HEADER) );
	inputHeader.ordinal = ChangeEndiannessDWord( TPM_ORD_ContinueSelfTest );
	

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(UINT32);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_ContinueSelfTest: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}		
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_ContinueSelfTest(), error in executing TPM_ContinueSelfTest command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}

	return u8CmdStatus;
}


/*Description: Read current information regarding the TPM*/
UINT8 TPM_Read_Volatile_Flags()
{
	UINT8 u8CmdStatus = TCG_OK;
	
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_GETCAP_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_GETCAP_PARAMS_IN inputParams;
	TPM_1_2_STCLEAR_FLAGS *pTPMVolatileFlags;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	//UINT32 i = 0;
	//UINT32 inputBufferSize = 0;
	UINT8 *u8CommandInputBuffer = NULL;
	//UINT8 *pu8CommandInputBuffer = NULL;
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	
	//UINT8 u8CommandInputBuffer[ sizeof(TPM_1_2_CMD_HEADER) + sizeof(TPM_1_2_GETCAP_PARAMS_IN) ] = {0}; 	
	//UINT8 u8CommandOutputBuffer[ sizeof(TPM_1_2_RET_HEADER) + sizeof(TPM_1_2_GETCAP_OUT) ] = {0}; 
	
	//u8CmdStatus = ExecuteTPMCmd(pu8TPMCmdInBuff, sizeof(TPM_GetCapability_In), pu8TPMCmdOutBuff, sizeof(TPM_GetCapability_Out));
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_GetCapability_In));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_GetCapability);
	inputParams.capArea = ChangeEndiannessDWord(TPM_CAP_FLAG);
	inputParams.subCapSize = ChangeEndiannessDWord(sizeof(UINT32));
	inputParams.subCap = ChangeEndiannessDWord(TPM_CAP_FLAG_VOLATILE);

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_PARAMS_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_Read_Volatile_Flags: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		
		//CopyMem(pu8TPMCmdOutBuff, u8CommandOutputBuffer, sizeof(TPM_1_2_RET_HEADER) + sizeof(TPM_1_2_GETCAP_OUT));
		
		pTPMVolatileFlags = (TPM_1_2_STCLEAR_FLAGS *) outputResponse.resp;
		Print(L"    deactivated                    = %X\n", pTPMVolatileFlags->deactivated);
		Print(L"    disableForceClear              = %X\n", pTPMVolatileFlags->disableForceClear);
		Print(L"    physicalPresence               = %X\n", pTPMVolatileFlags->physicalPresence);
		Print(L"    physicalPresenceLock           = %X\n", pTPMVolatileFlags->physicalPresenceLock);
		Print(L"    bGlobalLock                    = %X\n", pTPMVolatileFlags->bGlobalLock);	
		
		
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_GetCapability_cmd(), error in executing TPM_GetCapability command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}

	return u8CmdStatus;
}

/*Description: Read current information regarding the TPM*/
UINT8 TPM_Read_Non_Volatile_Flags()
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_GETCAP_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_GETCAP_PARAMS_IN inputParams;
	TPM_1_2_PERMANENT_FLAGS *pTPMNonVolatileFlags;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT8 *u8CommandInputBuffer = NULL;
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	//UINT8 u8CommandOutputBuffer[ sizeof(TPM_1_2_RET_HEADER) + sizeof(TPM_1_2_GETCAP_OUT) ] = {0}; 
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_GetCapability_In));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_GetCapability);
	inputParams.capArea = ChangeEndiannessDWord(TPM_CAP_FLAG);
	inputParams.subCapSize = ChangeEndiannessDWord(sizeof(UINT32));
	inputParams.subCap = ChangeEndiannessDWord(TPM_CAP_FLAG_PERMANENT);

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_PARAMS_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_Read_Non_Volatile_Flags: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		Print(L"  TPM Permanent Flags value:\n");
		pTPMNonVolatileFlags = (TPM_1_2_PERMANENT_FLAGS *) outputResponse.resp;
		Print(L"    disable                        = %X\n", pTPMNonVolatileFlags->disable);
		Print(L"    ownership                      = %X\n", pTPMNonVolatileFlags->ownership);//are we cabable of setting ownership
		Print(L"    deactivated                    = %X\n", pTPMNonVolatileFlags->deactivated);
		Print(L"    readPubek                      = %X\n", pTPMNonVolatileFlags->readPubek);
		Print(L"    disableOwnerClear              = %X\n", pTPMNonVolatileFlags->disableOwnerClear);
		Print(L"    allowMaintenance               = %X\n", pTPMNonVolatileFlags->allowMaintenance);
		Print(L"    physicalPresenceLifetimeLock   = %X\n", pTPMNonVolatileFlags->physicalPresenceLifetimeLock);
		Print(L"    physicalPresenceHWEnable       = %X\n", pTPMNonVolatileFlags->physicalPresenceHWEnable);
		Print(L"    physicalPresenceCMDEnable      = %X\n", pTPMNonVolatileFlags->physicalPresenceCMDEnable);
		//Print(L"    SECPUsed                       = %X\n", pTPMNonVolatileFlags->SECPUsed);
		//Print(L"    TPMPost                        = %X\n", pTPMNonVolatileFlags->TPMPost);
		//Print(L"    TPMPostLock                    = %X\n", pTPMNonVolatileFlags->TPMPostLock);
		Print(L"    FIPS                           = %X\n", pTPMNonVolatileFlags->FIPS);
		Print(L"    enableRevokeEK                 = %X\n", pTPMNonVolatileFlags->enableRevokeEK);
		Print(L"    nvLocked                       = %X\n", pTPMNonVolatileFlags->nvLocked);
		//Print(L"    readSPKPub                     = %X\n", pTPMNonVolatileFlags->readSPKPub);
		Print(L"    tpmEstablished                 = %X\n", pTPMNonVolatileFlags->tpmEstablished);
		
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_GetCapability_cmd(), error in executing TPM_GetCapability command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}

	return u8CmdStatus;
}

/*Description: Read current information regarding the TPM*/
UINT8 TPM_Read_NV_Index_List()
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_GETCAP_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_GETCAP_PARAMS_IN inputParams;
	TPM_1_2_PERMANENT_FLAGS *pTPMNonVolatileFlags;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	UINT32 i = 0;
	UINT32 *pTPMNVIndexList = NULL;
	//UINT8 u8CommandOutputBuffer[ sizeof(TPM_1_2_RET_HEADER) + sizeof(TPM_1_2_GETCAP_OUT) ] = {0}; 
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_GetCapability_In));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_GetCapability);
	inputParams.capArea = ChangeEndiannessDWord(TPM_CAP_NV_LIST);
	inputParams.subCapSize = ChangeEndiannessDWord(0x4); //ignored
	inputParams.subCap = 0; //ignored

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_PARAMS_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_Read_NV_Index_List: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		
		//List is an array of UINT32 with the number of elements = respSize/4.
		//debug Why are we getting a response size of 0 and no indexes when the call is successful
		//and I've provisioned with aux 2.
		//Print(L"Resp Size %d\n", outputResponse.respSize);
		outputResponse.respSize = ChangeEndiannessDWord(outputResponse.respSize);
		//Print(L"Resp Size %d\n", outputResponse.respSize);
		pTPMNVIndexList = (UINT32 *) outputResponse.resp;
		//DumpHexValue(L"TPM NV List", (UINT8 *)outputResponse.resp, sizeof(UINT32));
		Print(L"TPM Non Volatile Index List:\n");
		for ( i = 0; i < (outputResponse.respSize / 4); i++ ) {
			*pTPMNVIndexList = ChangeEndiannessDWord(*pTPMNVIndexList);
			Print(L"  Index %d: = 0x%X\n", i, *pTPMNVIndexList);
			pTPMNVIndexList++;
		}
		
		
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_GetCapability_cmd(), error in executing TPM_GetCapability command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}

	return u8CmdStatus;
}

/*Description: Read current information regarding the TPM*/
UINT8 TPM_Read_NV_Index_Info(IN UINT32 nvIndex, UINT32 *dataSize, BOOLEAN verbose, TPM_1_2_NV_DATA_PUBLIC *existingIndex)
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_GETCAP_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_GETCAP_PARAMS_IN inputParams;
	TPM_1_2_NV_DATA_PUBLIC *pTPMNonVolatileCap;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	//UINT8 u8CommandOutputBuffer[ sizeof(TPM_1_2_RET_HEADER) + sizeof(TPM_1_2_GETCAP_OUT) ] = {0}; 
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_GetCapability_In));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_GetCapability);
	inputParams.capArea = ChangeEndiannessDWord(TPM_CAP_NV_INDEX);
	inputParams.subCapSize = ChangeEndiannessDWord(0x4); //ignored
	inputParams.subCap = ChangeEndiannessDWord(nvIndex); //ignored

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_PARAMS_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
	
		if (outputHeader.returnCode != TPM_SUCCESS) {
				Print(L"TPM_Read_NV_Index_Info: Return Code  - 0x%08x\n", outputHeader.returnCode);
				return TCG_GENERAL_ERROR;
		}
		
		pTPMNonVolatileCap = (TPM_1_2_NV_DATA_PUBLIC *) outputResponse.resp;
		
		pTPMNonVolatileCap->Tag0 = ChangeEndiannessWord(pTPMNonVolatileCap->Tag0);
		pTPMNonVolatileCap->indexValue = ChangeEndiannessDWord(pTPMNonVolatileCap->indexValue);
		pTPMNonVolatileCap->readSizeofSelect = ChangeEndiannessWord(pTPMNonVolatileCap->readSizeofSelect);
		pTPMNonVolatileCap->writeSizeofSelect = ChangeEndiannessWord(pTPMNonVolatileCap->writeSizeofSelect);
		pTPMNonVolatileCap->Tag1 = ChangeEndiannessWord(pTPMNonVolatileCap->Tag1);
		pTPMNonVolatileCap->Attributes = ChangeEndiannessDWord(pTPMNonVolatileCap->Attributes);
		pTPMNonVolatileCap->dataSize = ChangeEndiannessDWord(pTPMNonVolatileCap->dataSize);
		
		CopyMem(existingIndex, (UINT8 *)outputResponse.resp, sizeof(TPM_1_2_NV_DATA_PUBLIC) );
		
		if ( verbose ) {
			Print(L"  0x%x Index Capabilities:\n", nvIndex);
			//pTPMNonVolatileCap = (TPM_1_2_NV_DATA_PUBLIC *) outputResponse.resp;
			Print(L"    tag                            = 0x%X\n", pTPMNonVolatileCap->Tag0);
			Print(L"    indexValue                     = 0x%X\n", pTPMNonVolatileCap->indexValue);
			Print(L"    readSizeofSelect               = 0x%X\n", pTPMNonVolatileCap->readSizeofSelect);
			Print(L"    readPcrSelect[0]               = 0x%X\n", pTPMNonVolatileCap->readPcrSelect[0]);
			Print(L"    readPcrSelect[1]               = 0x%X\n", pTPMNonVolatileCap->readPcrSelect[1]);
			Print(L"    readPcrSelect[2]               = 0x%X\n", pTPMNonVolatileCap->readPcrSelect[2]);
			Print(L"    readLocalityAtRelease          = 0x%X\n", pTPMNonVolatileCap->readLocalityAtRelease);
			DumpHexValue(L"readDigestAtRelease", (UINT8 *)&pTPMNonVolatileCap->readDigestAtRelease.Sha1, sizeof(DIGEST));
			Print(L"    writeSizeofSelect              = 0x%X\n", pTPMNonVolatileCap->writeSizeofSelect);
			Print(L"    writePcrSelect[0]              = 0x%X\n", pTPMNonVolatileCap->writePcrSelect[0]);
			Print(L"    writePcrSelect[1]              = 0x%X\n", pTPMNonVolatileCap->writePcrSelect[1]);
			Print(L"    writePcrSelect[2]              = 0x%X\n", pTPMNonVolatileCap->writePcrSelect[2]);
			Print(L"    writeLocalityAtRelease         = 0x%X\n", pTPMNonVolatileCap->writeLocalityAtRelease);
			DumpHexValue(L"readDigestAtRelease", (UINT8 *)&pTPMNonVolatileCap->writeDigestAtRelease.Sha1, sizeof(DIGEST));
			Print(L"    Tag1                           = 0x%X\n", pTPMNonVolatileCap->Tag1);
			Print(L"    Attributes                     = 0x%X\n", pTPMNonVolatileCap->Attributes);
			Print(L"    bReadSTClear                   = 0x%X\n", pTPMNonVolatileCap->bReadSTClear);
			Print(L"    bWriteSTClear                  = 0x%X\n", pTPMNonVolatileCap->bWriteSTClear);
			Print(L"    bWriteSDefine                  = 0x%X\n", pTPMNonVolatileCap->bWriteSDefine);
			Print(L"    dataSize                       = 0x%X\n", pTPMNonVolatileCap->dataSize);
		}
		*dataSize = pTPMNonVolatileCap->dataSize;
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_GetCapability_cmd(), error in executing TPM_GetCapability command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}
	return u8CmdStatus;
}



UINT8 TPM_PCR_Dump(CHAR16 *pcrFileName)
{
	UINT8 u8CmdStatus = TCG_OK;
	EFI_STATUS      Status; 
	EFI_FILE_HANDLE	FileHandle;
	UINT64			Attributes = 0;
	
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_PCRREAD_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_PCRREAD_PARAMS_IN inputParams;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	UINT32 i = 0;
	UINTN BufferSize = 20;
	
	Status = ShellOpenFileByName(pcrFileName, &FileHandle, EFI_FILE_MODE_READ | EFI_FILE_MODE_WRITE | EFI_FILE_MODE_CREATE, Attributes);
		
	if(Status != EFI_SUCCESS){
	
		switch (Status) {
			case EFI_INVALID_PARAMETER:
				Print(L"ERROR: EFI_INVALID_PARAMETER\n");
				break;
			case EFI_UNSUPPORTED:
				Print(L"ERROR: EFI_UNSUPPORTED\n");
				break;
			case EFI_NOT_FOUND:
				Print(L"ERROR: EFI_NOT_FOUND\n");
				break;
			case EFI_NO_MEDIA:
				Print(L"ERROR: EFI_NO_MEDIA\n");
				break;
			case EFI_MEDIA_CHANGED:
				Print(L"ERROR: EFI_MEDIA_CHANGED\n");
				break;
			case EFI_DEVICE_ERROR:
				Print(L"ERROR: EFI_DEVICE_ERROR\n");
				break;
			case EFI_VOLUME_CORRUPTED:
				Print(L"ERROR: EFI_VOLUME_CORRUPTED\n");
				break;
			case EFI_WRITE_PROTECTED:
				Print(L"ERROR: EFI_WRITE_PROTECTED\n");
				break;
			case EFI_ACCESS_DENIED:
				Print(L"ERROR: EFI_ACCESS_DENIED\n");
				break;
			case EFI_OUT_OF_RESOURCES:
				Print(L"ERROR: EFI_OUT_OF_RESOURCES\n");
				break;
			case EFI_VOLUME_FULL:
				Print(L"ERROR: EFI_VOLUME_FULL\n");
				break;
			default:
				Print(L"ERROR: Unknown error, cannot open file %s\n");
				break;
			}
		
		return Status;   
	} else {
	
		inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
		inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_PcrRead_In));
		inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_PcrRead);
		
		InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
		InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
		InputBuffers[1].Buffer = (UINT8 *)&inputParams;
		InputBuffers[1].Size = sizeof(TPM_1_2_PCRREAD_PARAMS_IN);
		
		OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
		OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
		OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
		OutputBuffers[1].Size = sizeof(TPM_1_2_PCRREAD_OUT);
		
		for ( i = 0; i < 24; i++ ) {
			inputParams.pcrIndex = ChangeEndiannessDWord(i);

			u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
			
			if(TCG_OK == u8CmdStatus){
				
				//pTPMNonVolatileFlags = (TPM_PERMANENT_FLAGS *) outputResponse.outDigest;
				Print(L"TPM PCR %d:", i);
				DumpBinary ((UINT8 *)outputResponse.outDigest, 20, 20, 2, 2);
				
				Status = ShellWriteFile( FileHandle,  &BufferSize, (void *)(UINT8 *)outputResponse.outDigest);
				if(Status != EFI_SUCCESS){
						Print(L"ERROR: Cannot write to file %s\n", pcrFileName);
						return EFI_SUCCESS;
				} else { 
					Print(L"INFO: %d bytes written.\n", BufferSize);
				}
				//close file
				Status = ShellCloseFile(&FileHandle);
		
			}
			else{
				errColor;
				DEBUG_LOG((L"ERROR : TPM_Read_PCRs(), error in executing TPM_Read_PCR command\n"));
				norColor;
				return TCG_GENERAL_ERROR;
			}
		}
	}
	return u8CmdStatus;
}

/*Description: Read current information regarding the TPM*/
UINT8 TPM_Read_PCRs()
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_PCRREAD_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_PCRREAD_PARAMS_IN inputParams;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	UINT32 i = 0;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_PcrRead_In));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_PcrRead);
	
	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_PCRREAD_PARAMS_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_PCRREAD_OUT);
	
	for ( i = 0; i < 24; i++ ) {
		inputParams.pcrIndex = ChangeEndiannessDWord(i);

		u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
		
		if(TCG_OK == u8CmdStatus){
			if (outputHeader.returnCode != TPM_SUCCESS) {
				Print(L"TPM_Read_PCRs: Return Code  - 0x%08x\n", outputHeader.returnCode);
				return TCG_GENERAL_ERROR;
			}
			//pTPMNonVolatileFlags = (TPM_PERMANENT_FLAGS *) outputResponse.outDigest;
			Print(L"TPM PCR %d:", i);
			DumpBinary ((UINT8 *)outputResponse.outDigest, 20, 20, 2, 2);
		}
		else{
			errColor;
			DEBUG_LOG((L"ERROR : TPM_Read_PCRs(), error in executing TPM_Read_PCR command\n"));
			norColor;
			return TCG_GENERAL_ERROR;
		}
	}
	return u8CmdStatus;
}	
/*Description: Read current information regarding the TPM*/


UINT8 TPM_PCR_Extend( UINT32 pcrNum, UINT8* inDigest )
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_PCR_EXTEND inputParams;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_PCRREAD_OUT outputResponse;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_1_2_CMD_HEADER) + sizeof(TPM_1_2_PCR_EXTEND));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_PCR_Extend);
	
	inputParams.pcrNum = pcrNum;
	CopyMem(inputParams.inDigest[0], inDigest, 20);
	
	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_PCR_EXTEND);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_PCRREAD_OUT);

	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_PCR_Extend: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		Print(L"PCR Extend: ");
		DumpBinary ((UINT8 *)outputResponse.outDigest, 20, 20, 2, 2);
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_PCR_Extend(), error in executing TPM_PCR_Extend command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}
	
	return u8CmdStatus;
}	

UINT8 TPM_PCR_Clear( UINT32 pcrNum )
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_PCRREAD_PARAMS_IN inputParams;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_PCRREAD_OUT outputResponse;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[1];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 1;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_1_2_CMD_HEADER) + sizeof(TPM_1_2_PCRREAD_PARAMS_IN));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_PCR_Reset);
	
	inputParams.pcrIndex = pcrNum;
	
	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_PCRREAD_PARAMS_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);

	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_PCR_Clear: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		Print(L"PCR Cleared");
	}
	else{
		errColor;
		DEBUG_LOG((L"ERROR : TPM_PCR_Clear(), error in executing TPM_PCR_Clear command\n"));
		norColor;
		return TCG_GENERAL_ERROR;
	}
	
	return u8CmdStatus;
}	



UINT8 TPM_NV_Read( UINT32 index, UINT32 dataSize, BOOLEAN verbose, TPM_1_3_NV_READVALUE_OUT *existingIndexData )
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_3_NV_READVALUE_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_NV_READVALUE_IN inputParams;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	UINT32* pu32Temp;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_1_2_CMD_HEADER) + sizeof(TPM_1_2_NV_READVALUE_IN) );
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_NV_ReadValue);
	inputParams.nvIndex = ChangeEndiannessDWord(index);
	inputParams.offset = ChangeEndiannessDWord(0);
	inputParams.dataSize = ChangeEndiannessDWord(dataSize);
	
	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_NV_READVALUE_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_3_NV_READVALUE_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_NV_Read: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		CopyMem(existingIndexData, &outputResponse, sizeof(TPM_1_3_NV_READVALUE_OUT) );
		if(verbose == TRUE){
			DumpHexValue (L"Index ", (UINT8 *) outputResponse.data, dataSize);
		}
	} else {
		errColor;
		Print(L"ERROR : TPM_NV_Read(), error in executing TPM_NV_Read command\n");
		norColor;
		return TCG_GENERAL_ERROR;
	}
	
	return u8CmdStatus;
}

//hard coded for testing
UINT8 TPM_NV_Define( TPM_DEFINE_SPACE *defSpace )
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TIS_BUFFER_INFO InputBuffers[1];
	TIS_BUFFER_INFO OutputBuffers[1];
	UINT32 NumInputBuffers = 1;
	UINT32 NumOutputBuffers = 1;
	//UINT32 nvindex = 0x50000003;
	//TPM_DEFINE_SPACE auxDefine;
	
	//UINT32 permissionAttributes = 0x2000;
	//UINT32 dataSize = 0x36; //54 bytes
	//inputParams.dataSize = ChangeEndiannessDWord(INDEX_AUX_SIZE); //UINT32
	//inputParams.dataSize = ((UINT32 *) ((UINT8 *)pNVAttributes + sizeof(TPM_1_2_NV_ATTRIBUTES) + sizeof(BOOLEAN) + sizeof(BOOLEAN) + sizeof(BOOLEAN)));

	//initDefineSpace(&auxDefine);
	//auxDefine.dataSize = INDEX_AUX_SIZE;
	//Print(L"defSpace->dataSize: %d\n", defSpace->dataSize);
	ChangeEndiannessTPMDefineSpace( defSpace );
	
	//InputBuffers[0].Buffer = (UINT8 *)&auxDefine;
	InputBuffers[0].Buffer = (UINT8 *)defSpace;
	InputBuffers[0].Size = sizeof(TPM_DEFINE_SPACE);
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_NV_Define: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
	}
	else{
		errColor;
		Print(L"ERROR : TPM_NV_Define(), error in executing TPM_NV_Define command\n");
		norColor;
		return TCG_GENERAL_ERROR;
	}
	
	return u8CmdStatus;
}


//hard coded for testing
UINT8 TPM_NV_Index_Delete()
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_AUTHDATA tpmAuthData = {0};
	TIS_BUFFER_INFO InputBuffers[1];
	TIS_BUFFER_INFO OutputBuffers[1];
	UINT32 NumInputBuffers = 1;
	UINT32 NumOutputBuffers = 1;
	//UINT32 nvindex = 0x50000003;
	EFI_STATUS      Status; 
	CHAR16 *Prompt = L"> ";
	CHAR16 **Response = AllocateZeroPool(256);
	UINT64 index;
	TPM_DEFINE_SPACE auxDefine;
	
	Print(L"Please enter index address to delete e.g. 0x50000003 ");
	Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
	Status = ShellConvertStringToUint64( *Response, &index, /*force hex*/ TRUE, /*stop at space*/ TRUE );
			
	initDefineSpace(&auxDefine);
	auxDefine.indexValue = (UINT32) index;
	auxDefine.dataSize = 0;
	ChangeEndiannessTPMDefineSpace(&auxDefine);
	
	InputBuffers[0].Buffer = (UINT8 *)&auxDefine;
	InputBuffers[0].Size = sizeof(TPM_DEFINE_SPACE);
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_NV_Define: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
	}
	else{
		errColor;
		Print(L"ERROR : TPM_NV_Define(), error in executing TPM_NV_Define command\n");
		norColor;
		return TCG_GENERAL_ERROR;
	}
	
	return u8CmdStatus;
}

//hard coded to write a ps policy for testing
UINT8 TPM_NV_Write( UINT32 nvIndex, UINT32 indexOffset, UINT32 numBytes, UINT8 *buffer )
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_NV_WRITEVALUE_IN inputParams;
	//UINT8 psPolicyData[54] = {0x02,0x02,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20 };
	TIS_BUFFER_INFO InputBuffers[3];
	TIS_BUFFER_INFO OutputBuffers[1];
	UINT32 NumInputBuffers = 3;
	UINT32 NumOutputBuffers = 1;
	//UINT64 nvindex = 0;
	//UINT32 dataSize = 54; //54 bytes
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof (TPM_1_2_CMD_HEADER) + sizeof(TPM_1_2_NV_WRITEVALUE_IN) + numBytes);//ChangeEndiannessDWord(paramSize);
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_NV_WriteValue);

	inputParams.nvIndex = ChangeEndiannessDWord((UINT32)nvIndex); //UINT32
	inputParams.offset = 0;//ChangeEndiannessDWord(nvindex); //UINT32
	inputParams.dataSize = ChangeEndiannessDWord(numBytes); // UINT32

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_NV_WRITEVALUE_IN); // -1 because  UINT8 data[1]; in the struct is variable size and included below
	InputBuffers[2].Buffer = (UINT8 *)buffer;
	InputBuffers[2].Size = numBytes;
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_NV_Write: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
	}
	else{
		errColor;
		Print(L"ERROR : TPM_NV_Write(), error in executing TPM_NV_Write command\n");
		norColor;
	}
	
	return u8CmdStatus;
}

UINT8 TPM_Read_Pubek( TPM_1_2_PUBKEY_OUT *pubKey )
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_KEY_PARMS * pTpmKeyParms;
	TPM_1_2_PUBKEY_OUT	outputResponse;
	UINT8 antiReplay[20] = {0};
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	UINT32* pu32Temp;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_1_2_CMD_HEADER) + sizeof(antiReplay) );
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_ReadPubek);
	
	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)antiReplay;
	InputBuffers[1].Size = sizeof(antiReplay);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_PUBKEY_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_Read_Pubek: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		//Print(L"pubKeyLength: %d \n", outputResponse.pubKeyLength);
		//DumpHexValue (L"pubKey ", (UINT8 *) outputResponse.pubKey, 256);
		//DumpHexValue (L"checksum ", (UINT8 *) outputResponse.checksum, 20);
		CopyMem(pubKey, &outputResponse, sizeof(TPM_1_2_PUBKEY_OUT));
		//DumpHexValue (L"pubKey ", (UINT8 *) pubKey->pubKey, 256);
		Print(L"pubKey->pubKeyLength: %d\n", ChangeEndiannessDWord(pubKey->pubKeyLength));
		Print(L"pubKey->keyParams.numPrimes: %d\n", ChangeEndiannessDWord(pubKey->keyParams.numPrimes));
		Print(L"pubKey->keyParams.exponentSize: %d\n", ChangeEndiannessDWord(pubKey->keyParams.exponentSize));
		//DumpHexValue (L"pubKey->keyParams.exponent: ", (UINT8 *)(pubKey->keyParams.exponent), ChangeEndiannessDWord(pubKey->keyParams.exponentSize));

	} else {
		errColor;
		Print(L"ERROR : TPM_Read_Pubek(), error in executing Read EK command. Error code: 0x%x \n", u8CmdStatus);
		norColor;
		return TCG_GENERAL_ERROR;
	}
	
	return u8CmdStatus;
}

UINT8 TPM_OIAP( UINT32 *authHandle, UINT8 *nonceEven )
{
	UINT8 				u8CmdStatus = TCG_OK;
	TPM_1_2_CMD_HEADER 	inputHeader;
	TPM_1_2_RET_HEADER 	outputHeader;
	TPM_1_2_OIAP_OUT	outputResponse;
	
	TIS_BUFFER_INFO InputBuffers[1];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 1;
	UINT32 NumOutputBuffers = 2;
	UINT32* pu32Temp;
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord( sizeof(TPM_1_2_CMD_HEADER) );
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_OIAP);
	
	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_OIAP_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	
	if(TCG_OK == u8CmdStatus){
		if (outputHeader.returnCode != TPM_SUCCESS) {
			Print(L"TPM_OIAP: Return Code  - 0x%08x\n", outputHeader.returnCode);
			return TCG_GENERAL_ERROR;
		}
		//Print(L"authHandle: %d \n", outputResponse.authhandle);
		//DumpHexValue (L"nonceEven ", (UINT8 *) outputResponse.nonceEven, 20);
		*authHandle =  outputResponse.authhandle;
		CopyMem(nonceEven, (UINT8 *) outputResponse.nonceEven, 20);
	} else {
		errColor;
		Print(L"ERROR : TPM_OIAP(), error in executing TPM_OIAP\n");
		norColor;
		return TCG_GENERAL_ERROR;
	}
	
	return u8CmdStatus;
}


void changeEndiannessTakeOwnerShip( TPM_1_2_TAKE_OWNERSHIP *inputParams )
{				
	inputParams->protocolID = ChangeEndiannessWord(inputParams->protocolID);					//2	
	inputParams->encOwnerAuthSize = ChangeEndiannessDWord(inputParams->encOwnerAuthSize); 		//4
	inputParams->encSrkAuthSize = ChangeEndiannessDWord(inputParams->encSrkAuthSize); 			//4
	
	//tpm key
	//inputParams->tpmKey.major = 0x1;															//1
	//inputParams->tpmKey.minor = 0x1;															//1
	//inputParams->tpmKey.revMajor = 0;															//1
	//inputParams->tpmKey.revMinor = 0;															//1
	
	inputParams->tpmKey.keyUsage = ChangeEndiannessWord(inputParams->tpmKey.keyUsage);			//2
	inputParams->tpmKey.keyFlags = ChangeEndiannessDWord(inputParams->tpmKey.keyFlags);			//4
	//inputParams->tpmKey.authDataUsage = 0; 													//1
	inputParams->tpmKey.algorithmID = ChangeEndiannessDWord(inputParams->tpmKey.algorithmID);	//4
	inputParams->tpmKey.encScheme = ChangeEndiannessWord(inputParams->tpmKey.encScheme);		//2
	inputParams->tpmKey.sigScheme = ChangeEndiannessWord(inputParams->tpmKey.sigScheme); 		//2
	inputParams->tpmKey.keyParmSize = ChangeEndiannessDWord(inputParams->tpmKey.keyParmSize);	//4
	inputParams->tpmKey.RsaKeyLength = ChangeEndiannessDWord(inputParams->tpmKey.RsaKeyLength);	//4
	inputParams->tpmKey.numPrimes = ChangeEndiannessDWord(inputParams->tpmKey.numPrimes);		//4
	inputParams->tpmKey.exponentSize = ChangeEndiannessDWord(inputParams->tpmKey.exponentSize);	//4
	inputParams->tpmKey.PCRInfoSize = ChangeEndiannessDWord(inputParams->tpmKey.PCRInfoSize);	//4
	inputParams->tpmKey.keyLength = ChangeEndiannessDWord(inputParams->tpmKey.keyLength);		//4
	//inputParams->key[256];						// UINT8 256
	inputParams->tpmKey.encDataSize = ChangeEndiannessDWord(inputParams->tpmKey.encDataSize);	//4
	inputParams->authHandle = ChangeEndiannessDWord(inputParams->authHandle);					//4
	//inputParams->nonceOdd[20];		//UINT8
	//inputParams->continueAuthSession;	//UINT8
	//inputParams->ownerAuth[20];		//UINT8
}

void fillSha1DataBuffer( UINT8 *Data, TPM_1_2_CMD_HEADER *inputHeader, TPM_1_2_TAKE_OWNERSHIP *inputParams)
{
	CopyMem(Data, &inputHeader->ordinal, 4);				//4
	Data += 4;
	CopyMem(Data, &inputParams->protocolID, 2);				//2	
	Data += 2;
	CopyMem(Data, &inputParams->encOwnerAuthSize, 4);		//4
	Data += 4;
	CopyMem(Data, &inputParams->encSrkAuthSize, 4);			//4
	Data += 4;
	//tpm key
	CopyMem(Data, &inputParams->tpmKey.major, 1);			//1	
	Data += 1;
	CopyMem(Data, &inputParams->tpmKey.minor, 1);			//1
	Data += 1;
	CopyMem(Data, &inputParams->tpmKey.revMajor, 1);		//1
	Data += 1;
	CopyMem(Data, &inputParams->tpmKey.revMinor, 1);		//1
	Data += 1;
	CopyMem(Data, &inputParams->tpmKey.keyUsage, 2);		//2
	Data += 2;
	CopyMem(Data, &inputParams->tpmKey.keyFlags, 4);		//4
	Data += 4;
	CopyMem(Data, &inputParams->tpmKey.authDataUsage, 1);	//1
	Data += 1;							
	CopyMem(Data, &inputParams->tpmKey.algorithmID, 4);		//4
	Data += 4;
	CopyMem(Data, &inputParams->tpmKey.encScheme, 2);		//2
	Data += 2;
	CopyMem(Data, &inputParams->tpmKey.sigScheme, 2); 		//2
	Data += 2;
	CopyMem(Data, &inputParams->tpmKey.keyParmSize, 4);		//4
	Data += 4;
	CopyMem(Data, &inputParams->tpmKey.RsaKeyLength, 4);	//4
	Data += 4;
	CopyMem(Data, &inputParams->tpmKey.numPrimes, 4);		//4
	Data += 4;
	CopyMem(Data, &inputParams->tpmKey.exponentSize, 4);	//4
	Data += 4;
	CopyMem(Data, &inputParams->tpmKey.PCRInfoSize, 4);		//4
	Data += 4;
	CopyMem(Data, &inputParams->tpmKey.keyLength, 4);		//4
	Data += 4;
	CopyMem(Data, inputParams->tpmKey.key, 256);			// UINT8 256
	Data += 256;
	CopyMem(Data, &inputParams->tpmKey.encDataSize, 4);		//4
	Data += 4;
}


UINT8 TPM_TakeOwnership( UINT8 *ownerAuthValue, UINT8 *encOwnerAuthEK, UINT8 *encSrkAuthEK, UINT32 authHandle, UINT8 *nonceOdd, UINT8 *nonceEven, TPM_1_2_PUBKEY_OUT *pubKey )
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_TAKE_OWNERSHIP inputParams;
	TPM_1_2_PUBKEY_OUT	outputResponse;
	TPM_1_2_KEY tpmKey;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	UINT32* pu32Temp;
	
	UINTN hmacSha1GetContextSize = 0;
	UINTN sha1DataSize = 605;
	UINT8 sha1Data[605];
	UINT8 sha1DataHash[20];
	
	VOID *HmacSha1Context = NULL;
	BOOLEAN hmacStatus;
	UINTN hmacDataSize = 61;
	UINT8 hmacData[61];

	UINT8 authValue[20];
	
	Print(L"Take ownership command args\n");
	DumpHexValue (L"ownerAuthValue: ", (UINT8 *) ownerAuthValue, 20);
	DumpHexValue (L"encOwnerAuthEK: ", (UINT8 *) encOwnerAuthEK, 256);
	DumpHexValue (L"encSrkAuthEK: ", (UINT8 *) encSrkAuthEK, 256);
	Print(L"authHandle: \n", authHandle );
	DumpHexValue (L"nonceOdd: ", (UINT8 *) nonceOdd, 20);
	DumpHexValue (L"nonceEven: ", (UINT8 *) nonceEven, 20);
	DumpHexValue (L"pubKey ", (UINT8 *) pubKey->pubKey, 256);
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_1_2_CMD_HEADER) + sizeof(TPM_1_2_TAKE_OWNERSHIP) );
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_ReadPubek);
	
	inputParams.protocolID = 0x5;					//2
	inputParams.encOwnerAuthSize = 0x100;			//4
	CopyMem(inputParams.encOwnerAuth, encOwnerAuthEK, 256);
	inputParams.encSrkAuthSize = 0x100;				//4
	CopyMem(inputParams.encSrkAuth, encSrkAuthEK, 256);
	
	//tpm key
	inputParams.tpmKey.major = 0x1;					//1
	inputParams.tpmKey.minor = 0x1;					//1
	inputParams.tpmKey.revMajor = 0;				//1
	inputParams.tpmKey.revMinor = 0;				//1
	
	inputParams.tpmKey.keyUsage = 0x0011;			//2
	inputParams.tpmKey.keyFlags = 0;
	inputParams.tpmKey.authDataUsage = 0; 			//TPM_AUTH_NEVER
	inputParams.tpmKey.algorithmID = 1; 			//TPM_ALG_RSA
	inputParams.tpmKey.encScheme = 3;
	inputParams.tpmKey.sigScheme = 1; 				//TPM_SS_NONE
	inputParams.tpmKey.keyParmSize = 0x0000000C;	//
	inputParams.tpmKey.RsaKeyLength = 0x00000800;	//
	inputParams.tpmKey.numPrimes = 2;				//
	inputParams.tpmKey.exponentSize = 0;			//
	inputParams.tpmKey.PCRInfoSize = 0;				//
	inputParams.tpmKey.keyLength = 256;				//
	ZeroMem(inputParams.tpmKey.key, 256);			//
	CopyMem(inputParams.tpmKey.key, pubKey->pubKey, 256);
	inputParams.tpmKey.encDataSize = 0;				//
	
	inputParams.authHandle = authHandle;			//value from TPM_OIAP response
	ZeroMem(inputParams.nonceOdd, 20);
	CopyMem(inputParams.nonceOdd, nonceOdd, 20);
	inputParams.continueAuthSession = 0; 			//false - don't continue
	
	//change input to big endian
	changeEndiannessTakeOwnerShip( &inputParams );
	
	//sha1 parameters to Data buffer
	fillSha1DataBuffer( &sha1Data, &inputHeader, &inputParams);
	
	//debug
	DumpHexValue (L"sha1Data: ", (UINT8 *) &sha1Data, 646);
	Print(L"Verify sha1data\n");
	Print(L"%x", inputHeader.ordinal );
	Print(L"%x", inputParams.protocolID );					//2
	Print(L"%x", inputParams.encOwnerAuthSize );			//4
	DumpHexValue (L"", (UINT8 *) inputParams.encOwnerAuth, 256);
	Print(L"%x", inputParams.encSrkAuthSize );				//4
	DumpHexValue (L"", (UINT8 *) inputParams.encSrkAuth, 256);
	
	//tpm key
	Print(L"%x", inputParams.tpmKey.major);					//1
	Print(L"%x", inputParams.tpmKey.minor );					//1
	Print(L"%x", inputParams.tpmKey.revMajor );				//1
	Print(L"%x", inputParams.tpmKey.revMinor );				//1
	
	Print(L"%x", inputParams.tpmKey.keyUsage );			//2
	Print(L"%x", inputParams.tpmKey.keyFlags );
	Print(L"%x", inputParams.tpmKey.authDataUsage ); 			//TPM_AUTH_NEVER
	Print(L"%x", inputParams.tpmKey.algorithmID ); 			//TPM_ALG_RSA
	Print(L"%x", inputParams.tpmKey.encScheme );
	Print(L"%x", inputParams.tpmKey.sigScheme ); 				//TPM_SS_NONE
	Print(L"%x", inputParams.tpmKey.keyParmSize );	//
	Print(L"%x", inputParams.tpmKey.RsaKeyLength );	//
	Print(L"%x", inputParams.tpmKey.numPrimes );				//
	Print(L"%x", inputParams.tpmKey.exponentSize );			//
	Print(L"%x", inputParams.tpmKey.PCRInfoSize );				//
	Print(L"%x", inputParams.tpmKey.keyLength );				//
	DumpHexValue (L"", (UINT8 *) inputParams.encSrkAuth, 256);
	Print(L"%x", inputParams.tpmKey.encDataSize );				//
	
	
	//sha1 the inParams
	Sha1( (UINT8 *) sha1Data, sha1DataSize, (UINT32 *)sha1DataHash);
	
	//create owner auth; hmac of the take ownership message, ownerAuthValue, and nonceEven
	//hmacSha1GetContextSize = HmacSha1GetContextSize( );
	if ( hmacSha1GetContextSize == 0 ) {
		Print(L"hmacSha1GetContextSize = 0\n");
	}
	//HmacSha1Context = AllocateZeroPool (hmacSha1GetContextSize);
	//hmacStatus = HmacSha1Init ( OUT HmacSha1Context, IN ownerAuthValue, IN 20 );
	if (!hmacStatus) {
		Print (L"HmacSha1Init [Fail]");
	}
	
	//HMAC( SHA1( inparams) + OIAP.nonceEven + nonceOdd + continueAuthSession )
	CopyMem(hmacData[0], sha1DataHash, 20);
	
	CopyMem(hmacData[20], nonceEven, 20);						//20
	
	CopyMem(hmacData[40], nonceOdd, 20);						//20
	
	CopyMem(hmacData[60], inputParams.continueAuthSession, 1);	//1
	
	DumpHexValue (L"hmacData: ", (UINT8 *) hmacData, 61);
	
	//hmacStatus = HmacSha1Update ( IN OUT HmacSha1Context, IN (VOID *)hmacData, IN hmacDataSize );
	if (!hmacStatus) {
		Print (L"HmacSha1Update [Fail]");
	}
	//hmacStatus = HmacSha1Final ( IN OUT HmacSha1Context, OUT authValue );
	if (!hmacStatus) {
		Print (L"HmacSha1Final [Fail]");
	}
	
	//add authValueto the inputparams
	ZeroMem(inputParams.ownerAuth, 20); //hmac
	CopyMem(&inputParams.ownerAuth, authValue, 20);
	
	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_TAKE_OWNERSHIP);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_PUBKEY_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	
	if(TCG_OK == u8CmdStatus){
		Print(L"pubKeyLength: %d \n", outputResponse.pubKeyLength);
		DumpHexValue (L"pubKey ", (UINT8 *) outputResponse.pubKey, 256);
		DumpHexValue (L"checksum ", (UINT8 *) outputResponse.checksum, 20);
	} else {
		errColor;
		Print(L"ERROR : TPM_Read_Pubek(), error in executing Read EK command\n");
		norColor;
	}
	
	return u8CmdStatus;
}


/*****************************End TPM Helper Functions****************************************************/

BOOLEAN verifyTPMPresence()
{
	TPM *pTPM = (TPM *)TPM_BASE_0; // 0xFED40000
	if (0xFF == pTPM->ACCESS){
		errColor;
		//Print(L"ERROR: TPM is not found or is broken. Access register value = 0xFF.\n\n"); 
		norColor;
		return FALSE;
	} 
	return TRUE;
}

BOOLEAN verifyTPMLock()
{
	UINT8 u8CmdStatus = TCG_OK;
	TPM_1_2_RET_HEADER outputHeader;
	TPM_1_2_GETCAP_OUT outputResponse;
	TPM_1_2_CMD_HEADER inputHeader;
	TPM_1_2_GETCAP_PARAMS_IN inputParams;
	TPM_1_2_PERMANENT_FLAGS *pTPMNonVolatileFlags;
	TIS_BUFFER_INFO InputBuffers[2];
	TIS_BUFFER_INFO OutputBuffers[2];
	UINT8 *u8CommandInputBuffer = NULL;
	UINT32 NumInputBuffers = 2;
	UINT32 NumOutputBuffers = 2;
	//UINT8 u8CommandOutputBuffer[ sizeof(TPM_1_2_RET_HEADER) + sizeof(TPM_1_2_GETCAP_OUT) ] = {0}; 
	
	inputHeader.tag = ChangeEndiannessWord(TPM_TAG_RQU_COMMAND);
	inputHeader.paramSize = ChangeEndiannessDWord(sizeof(TPM_GetCapability_In));
	inputHeader.ordinal = ChangeEndiannessDWord(TPM_ORD_GetCapability);
	inputParams.capArea = ChangeEndiannessDWord(TPM_CAP_FLAG);
	inputParams.subCapSize = ChangeEndiannessDWord(sizeof(UINT32));
	inputParams.subCap = ChangeEndiannessDWord(TPM_CAP_FLAG_PERMANENT);

	InputBuffers[0].Buffer = (UINT8 *)&inputHeader;
	InputBuffers[0].Size = sizeof(TPM_1_2_CMD_HEADER);
	InputBuffers[1].Buffer = (UINT8 *)&inputParams;
	InputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_PARAMS_IN);
	
	OutputBuffers[0].Buffer = (UINT8 *)&outputHeader;
	OutputBuffers[0].Size = sizeof(TPM_1_2_RET_HEADER);
	OutputBuffers[1].Buffer = (UINT8 *)&outputResponse;
	OutputBuffers[1].Size = sizeof(TPM_1_2_GETCAP_OUT);
	
	u8CmdStatus = ExecuteTPMCmdSGL(NumInputBuffers, InputBuffers, NumOutputBuffers, OutputBuffers );
	
	if(TCG_OK == u8CmdStatus){
		
		pTPMNonVolatileFlags = (TPM_1_2_PERMANENT_FLAGS *) outputResponse.resp;

		if ( pTPMNonVolatileFlags->nvLocked != 0 ) {
			return TRUE;
		} else {
			return FALSE;
		}
		
	}
	else{
		errColor;
		Print(L"ERROR : verifyTPMLock(), error in executing TPM_GetCapability command\n");
		norColor;
	}
	
	return FALSE;
}


/*
BOOLEAN verifyTPMEnabled()
{
	TPM_1_2_PERMANENT_FLAGS *pTPMPermntFlags = NULL;
	readTPMPermanentFlags();
	
	if (0 == gTPM_GetCapability_Permanent_flag.returnCode){  	
		pTPMPermntFlags = (TPM_1_2_PERMANENT_FLAGS *)gTPM_GetCapability_Permanent_flag.resp;
		
		if(1 == pTPMPermntFlags->disable){
			errColor;
			//Print(L"ERROR: TPM is disabled.\n");
			norColor;
			return FALSE;
		} else{
			//Print(L"INFO: TPM is enabled.\n");
			return TRUE;
		} 	
	} 
	return FALSE;
}
*/


void DisplayTPMStatus()
{
	
	TPM *pTPM = (TPM *)TPM_BASE_0; // 0xFED40000
	UINT32 *pTPMStatus = TPM_BASE_0 + 0x18;
	//CpuDeadLoop();
	if (0xFF == pTPM->ACCESS){
		errColor;
		Print(L"ERROR: TPM is not found or is broken. Access register value = 0xFF.\n\n"); 
		norColor;
		return;
	}

	Print(L"TPM Access Register [0xFED4-0000] <value=0x%X>\n", pTPM->ACCESS);
	Print(L"   Bit7  %X     tpmRegValidSts (register valid?)\n", ((pTPM->ACCESS & 0x80) == 0) ? 0 : 1);
	Print(L"   Bit6  %X     Reserved\n", ((pTPM->ACCESS & 0x40) == 0) ? 0 : 1);
	Print(L"   Bit5  %X     activeLocality (active locality?)\n", ((pTPM->ACCESS & 0x20) == 0) ? 0 : 1);
	Print(L"   Bit4  %X     beenSeized     (seized by higher locality?)\n", ((pTPM->ACCESS & 0x10) == 0) ? 0 : 1);
	Print(L"   Bit3  %X     Seize (Write only bit)\n", ((pTPM->ACCESS & 0x8) == 0) ? 0 : 1);
	Print(L"   Bit2  %X     pendingRequest (another locality requesting use?)\n", ((pTPM->ACCESS & 0x4) == 0) ? 0 : 1);
	Print(L"   Bit1  %X     requestUse (This locality is requesting to use TPM?)\n", ((pTPM->ACCESS & 0x2) == 0) ? 0 : 1);
	Print(L"   Bit0  %X     tpmEstablishment (Successful secure launch?)\n", ((pTPM->ACCESS & 0x1) == 0) ? 0 : 1);
	
	Print(L"TPM Status  Register [0xFED4-0018] <value=0x%X>\n", *pTPMStatus);
	Print(L"   23:8  %04x     burstCount (number of bytes that the TPM can read/write)\n", (*pTPMStatus >> 8) & 0x0000FFFF);
	Print(L"   Bit7  %X     stsValid (TPM_STS_x.dataAvail and TPM_STS_x.Expect are valid)\n", ((*pTPMStatus & 0x80) == 0) ? 0 : 1);
	Print(L"   Bit6  %X     commandReady (TPM ready to receive command?)\n", ((*pTPMStatus & 0x40) == 0) ? 0 : 1);
	Print(L"   Bit5  %X     tpmGo (Write only)\n", ((*pTPMStatus & 0x20) == 0) ? 0 : 1);
	Print(L"   Bit4  %X     dataAvail (response data available?)\n", ((*pTPMStatus & 0x10) == 0) ? 0 : 1);
	Print(L"   Bit3  %X     Expect (TPM expects more data for a command?) \n", ((*pTPMStatus & 0x8) == 0) ? 0 : 1);
	Print(L"   Bit2  %X     selfTestDone (TPM completed all self-test actions?)\n", ((*pTPMStatus & 0x4) == 0) ? 0 : 1);
	Print(L"   Bit1  %X     responseRetry (Write only)\n", ((*pTPMStatus & 0x2) == 0) ? 0 : 1);
	Print(L"   Bit0  %X     reserved\n", ((*pTPMStatus & 0x1) == 0) ? 0 : 1);


}


UINT8 get_TPM_Auth_Value( UINT8 authType, UINT8 *AuthValue )
{
	EFI_STATUS      Status; 
	EFI_FILE_HANDLE	FileHandle;
	UINTN 			Size = 20;
	CHAR16			TagBuffer[2];
	CHAR16 			fname[256];
	CHAR16 			writeBuf[256];
	UINTN   		BufferSize = sizeof(writeBuf);
	UINT64			Attributes = 0;
	CHAR16 **Response = AllocateZeroPool(256);
	CHAR16 *Prompt = L"Please enter an auth password> ";
	UINT32 MsgLen = 0;
	UINT8 HashOut[20];
	
	if ( authType == TPM_OWNER_AUTH ) {
		StrCpy(fname, L"OwnerAuth.bin");
	} else {
		StrCpy(fname, L"SrkAuth.bin");
	}
	
	Status = ShellOpenFileByName( fname, &FileHandle, EFI_FILE_MODE_READ, Attributes );
	if(Status == EFI_SUCCESS){
		//read auth value
		
		Status = ShellReadFile( &FileHandle, &Size, (void *)HashOut);
		//DumpHexValue (L"Auth Value ", (UINT8 *) HashOut, Size);
  
	} else if (Status == EFI_NOT_FOUND){
		Print(L"ERROR: EFI_NOT_FOUND, creating new auth file...\n");
		Status = ShellOpenFileByName( fname, &FileHandle, EFI_FILE_MODE_READ | EFI_FILE_MODE_WRITE | EFI_FILE_MODE_CREATE, Attributes );
		if(Status != EFI_SUCCESS){
			return Status;
		}
		
		Status = ShellPromptForResponse( IN ShellPromptResponseTypeFreeform,  IN Prompt, IN OUT Response );
		
		Print(L"*response: %s \n", *Response);
		if (StrCmp(*Response, NULL) == 0 ) {
			return Status;
		}
		MsgLen = 2 * StrLen(*Response);
		Sha1( (UINT8 *) *Response, MsgLen, (UINT32 *)HashOut);
		
		DumpHexValue (L"Auth Value ", (UINT8 *) HashOut, Size);
		
		ShellWriteFile(FileHandle, &Size, (void *)HashOut);
		
	} else {
		Print(L"ERROR: Unknown error, cannot open file %s\n");
		return Status;
	}
		
	CopyMem(AuthValue, (UINT8 *) HashOut, 20);

	return Status;
}

CHAR8 *mystrtok(CHAR16* string, const CHAR16 *delim)
{
	
    CHAR16 *p;
	CHAR16 *del;
	//CpuDeadLoop();
    if(string != NULL) {
		//Print(L"string not null: %s\n", string);
        ptr = string;
        p = string;
    } else {
		//Print(L"string null: %s\n", string);
        if(*ptr == L'\0') {
            return NULL;
		}
        p = ptr;
    }

    while(*ptr != L'\0') {
        del = delim;
		//Print(L"*del: %s\n", *del);
        while(*del != L'\0') {
			//Print(L"ptr[0]: %s\n", *ptr[0]);
			//Print(L"ptr[0]: %s\n", &ptr[0]);
			//Print(L"del: %s\n", del);
            if(*ptr == *del) {
                if(ptr == p) {
					//Print(L"ptr == p\n");
                    p++;
                    ptr++;
                } else {
                    *ptr = L'\0';
                    ptr++;

                    return p;
                }
            } else {
                del++;
            }
        }
        ptr++;
    }
    return p;
}
